@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020 - 2024 the ThorVG project. All rights reserved.
+ * Copyright (c) 2020 - 2026 ThorVG project. All rights reserved.
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,8 +22,7 @@
 
 #include "config.h"
 #include <cmath>
-#include <cstring>
-#include <memory.h>
+#include "tvgCommon.h"
 #include "tvgMath.h"
 #include "tvgStr.h"
 
@@ -56,13 +55,13 @@ namespace tvg {
  * No hexadecimal form supported
  * no sequence supported after NAN
  */
-float strToFloat(const char *nPtr, char **endPtr)
+float toFloat(const char *str, char **end)
 {
-    if (endPtr) *endPtr = (char *) (nPtr);
-    if (!nPtr) return 0.0f;
+    if (end) *end = (char *) (str);
+    if (!str) return 0.0f;
 
-    auto a = nPtr;
-    auto iter = nPtr;
+    auto a = str;
+    auto iter = str;
     auto val = 0.0f;
     unsigned long long integerPart = 0;
     int minus = 1;
@@ -88,7 +87,7 @@ float strToFloat(const char *nPtr, char **endPtr)
                 iter += 5;
             else goto error;
         }
-        if (endPtr) *endPtr = (char *) (iter);
+        if (end) *end = (char *) (iter);
         return (minus == -1) ? -INFINITY : INFINITY;
     }
 
@@ -96,7 +95,7 @@ float strToFloat(const char *nPtr, char **endPtr)
         if ((tolower(*(iter + 1)) == 'a') && (tolower(*(iter + 2)) == 'n')) iter += 3;
         else goto error;
 
-        if (endPtr) *endPtr = (char *) (iter);
+        if (end) *end = (char *) (iter);
         return (minus == -1) ? -NAN : NAN;
     }
 
@@ -165,7 +164,7 @@ float strToFloat(const char *nPtr, char **endPtr)
                 exponentPart = exponentPart * 10U + static_cast<unsigned int>(*iter - '0');
             }
         } else if (!isdigit(*(a - 1))) {
-            a = nPtr;
+            a = str;
             goto success;
         } else if (*iter == 0) {
             goto success;
@@ -191,43 +190,45 @@ float strToFloat(const char *nPtr, char **endPtr)
             exponentPart--;
         }
         val = (minus_e == -1) ? (val / scale) : (val * scale);
-    } else if ((iter > nPtr) && !isdigit(*(iter - 1))) {
-        a = nPtr;
+    } else if ((iter > str) && !isdigit(*(iter - 1))) {
+        a = str;
         goto success;
     }
 
 success:
-    if (endPtr) *endPtr = (char *)(a);
+    if (end) *end = (char *)a;
     if (!std::isfinite(val)) return 0.0f;
 
     return minus * val;
 
 error:
-    if (endPtr) *endPtr = (char *)(nPtr);
+    if (end) *end = (char *)str;
     return 0.0f;
 }
 
-char* strDuplicate(const char *str, size_t n)
+
+char* duplicate(const char *str, size_t n)
 {
     auto len = strlen(str);
     if (len < n) n = len;
 
-    auto ret = (char *) malloc(n + 1);
-    if (!ret) return nullptr;
+    auto ret = tvg::malloc<char>(n + 1);
     ret[n] = '\0';
 
-    return (char *) memcpy(ret, str, n);
+    return (char*)memcpy(ret, str, n);
 }
 
-char* strAppend(char* lhs, const char* rhs, size_t n)
+
+char* append(char* lhs, const char* rhs, size_t n)
 {
     if (!rhs) return lhs;
-    if (!lhs) return strDuplicate(rhs, n);
-    lhs = (char*)realloc(lhs, strlen(lhs) + n + 1);
+    if (!lhs) return duplicate(rhs, n);
+    lhs = tvg::realloc<char>(lhs, strlen(lhs) + n + 1);
     return strncat(lhs, rhs, n);
 }
 
-char* strDirname(const char* path)
+
+char* dirname(const char* path)
 {
     auto ptr = strrchr(path, '/');
 #ifdef _WIN32
@@ -235,7 +236,44 @@ char* strDirname(const char* path)
     if (ptr2) ptr = ptr2;
 #endif
     auto len = ptr ? size_t(ptr - path + 1) : SIZE_MAX;
-    return strDuplicate(path, len);
+    return duplicate(path, len);
+}
+
+
+char* filename(const char* path)
+{
+    const char* ptr = strrchr(path, '/');
+#ifdef _WIN32
+    auto ptr2 = strrchr(ptr ? ptr : path, '\\');
+    if (ptr2) ptr = ptr2;
+#endif
+    if (ptr) ++ptr;
+    else ptr = path;
+    auto dot = fileext(ptr);
+    auto len = (dot > ptr) ? (size_t)(dot - ptr - 1) : strlen(ptr);
+    return duplicate(ptr, len);
+}
+
+
+const char* fileext(const char* path)
+{
+    auto ext = path;
+    while (ext) {
+        auto p = strchr(ext, '.');
+        if (!p) break;
+        ext = p + 1;
+    }
+    return ext;
+}
+
+
+char* concat(const char* a, const char* b)
+{
+    auto len = strlen(a) + strlen(b) + 1;
+    auto ret = tvg::malloc<char>(len * sizeof(char));
+    strcpy(ret, a);
+    strcat(ret, b);
+    return ret;
 }
 
 }
