@@ -120,9 +120,6 @@ typedef enum
 #if JERRY_MODULE_SYSTEM
   ECMA_PARSE_INTERNAL_HAS_IMPORT_META = (1u << 18), /**< module has import.meta expression */
 #endif /* JERRY_MODULE_SYSTEM */
-#if JERRY_FUNCTION_TO_STRING
-  ECMA_PARSE_INTERNAL_HAS_4_BYTE_MARKER = (1u << 19), /**< source has 4 byte marker */
-#endif /* JERRY_FUNCTION_TO_STRING */
 #ifndef JERRY_NDEBUG
   /**
    * This flag represents an error in for in/of statements, which cannot be set
@@ -521,16 +518,9 @@ typedef uint8_t ecma_property_t; /**< ecma_property_types_t (3 bit) and ecma_pro
  */
 typedef struct
 {
-#if JERRY_CPOINTER_32_BIT
-  jmem_cpointer_t next_property_cp; /**< next cpointer */
-#endif /* JERRY_CPOINTER_32_BIT */
   ecma_property_t types[ECMA_PROPERTY_PAIR_ITEM_COUNT]; /**< two property type slot. The first represent
                                                          *   the type of this property (e.g. property pair) */
-#if JERRY_CPOINTER_32_BIT
-  uint16_t padding; /**< an unused value */
-#else /* !JERRY_CPOINTER_32_BIT */
   jmem_cpointer_t next_property_cp; /**< next cpointer */
-#endif /* JERRY_CPOINTER_32_BIT */
 } ecma_property_header_t;
 
 /**
@@ -548,11 +538,7 @@ typedef struct
 typedef union
 {
   ecma_value_t value; /**< value of a property */
-#if JERRY_CPOINTER_32_BIT
-  jmem_cpointer_t getter_setter_pair_cp; /**< cpointer to getter setter pair */
-#else /* !JERRY_CPOINTER_32_BIT */
   ecma_getter_setter_pointers_t getter_setter_pair; /**< getter setter pair */
-#endif /* JERRY_CPOINTER_32_BIT */
 } ecma_property_value_t;
 
 /**
@@ -863,22 +849,11 @@ typedef enum
  */
 #define ECMA_OBJECT_REF_ONE (1u << ECMA_OBJECT_REF_SHIFT)
 
-#if JERRY_CPOINTER_32_BIT
-
-/**
- * Type of the descriptor field of an object
- */
-typedef uint32_t ecma_object_descriptor_t;
-
-#else /* !JERRY_CPOINTER_32_BIT */
-
 /**
  * Type of the descriptor field of an object
  */
 typedef uint16_t ecma_object_descriptor_t;
 
-#endif /* JERRY_CPOINTER_32_BIT */
-
 /**
  * Bitmask for an ecma-object reference count field
  */
@@ -1075,7 +1050,7 @@ typedef struct
         ecma_value_t iterated_value; /**< for %Iterator%: [[IteratedObject]] property */
         ecma_value_t promise; /**< PromiseCapability[[Promise]] internal slot */
         ecma_value_t sync_iterator; /**< IteratorRecord [[Iterator]] internal slot for AsyncFromSyncIterator */
-        ecma_value_t spread_value; /**< for spread object: spreaded element */
+        ecma_value_t spread_value; /**< for spread object: spread element */
         int32_t tza; /**< TimeZone adjustment for date objects */
         uint32_t length; /**< length related property (e.g. length of ArrayBuffer) */
         uint32_t arguments_number; /**< for arguments: arguments number */
@@ -1439,11 +1414,7 @@ typedef enum
  * Maximum value of the immediate part of a direct magic string.
  * Must be compatible with the immediate property name.
  */
-#if JERRY_CPOINTER_32_BIT
-#define ECMA_DIRECT_STRING_MAX_IMM 0x07ffffff
-#else /* !JERRY_CPOINTER_32_BIT */
 #define ECMA_DIRECT_STRING_MAX_IMM 0x0000ffff
-#endif /* JERRY_CPOINTER_32_BIT */
 
 /**
  * Shift for direct string value part in ecma_value_t.
@@ -1770,11 +1741,7 @@ typedef struct
 /**
  * Container of an LCache entry identifier
  */
-#if JERRY_CPOINTER_32_BIT
-typedef uint64_t ecma_lcache_hash_entry_id_t;
-#else /* !JERRY_CPOINTER_32_BIT */
 typedef uint32_t ecma_lcache_hash_entry_id_t;
-#endif /* JERRY_CPOINTER_32_BIT */
 
 /**
  * Entry of LCache hash table
