@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2023 - 2024 the ThorVG project. All rights reserved.
+ * Copyright (c) 2023 - 2026 ThorVG project. All rights reserved.
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -20,45 +20,58 @@
  * SOFTWARE.
  */
 
-#include "tvgLottieLoader.h"
+#include "tvgStr.h"
+ #include "tvgLottieLoader.h"
 #include "tvgLottieModel.h"
 #include "tvgLottieParser.h"
 #include "tvgLottieBuilder.h"
-#include "tvgStr.h"
+#include "tvgCompressor.h"
 
 /************************************************************************/
 /* Internal Class Implementation                                        */
 /************************************************************************/
 
+LottieCustomSlot::~LottieCustomSlot()
+{
+    ARRAY_FOREACH(p, props) {
+        delete(p->prop);
+    }
+}
+
+
+bool LottieLoader::prepare()
+{
+    LottieParser parser(content, dirName, builder->expressions());
+    if (!parser.parse()) return false;
+    {
+        ScopedLock lock(key);
+        comp = parser.comp;
+    }
+    if (!comp) return false;
+    if (parser.slots) {
+        auto slotcode = gen(parser.slots, true);
+        apply(slotcode, true);
+        del(slotcode, true);
+        parser.slots = nullptr;
+    }
+    builder->build(comp);
+    release();
+    return true;
+}
+
+
 void LottieLoader::run(unsigned tid)
 {
-    //update frame
-    if (comp) {
-        builder->update(comp, frameNo);
-    //initial loading
-    } else {
-        LottieParser parser(content, dirName);
-        if (!parser.parse()) return;
-        {
-            ScopedLock lock(key);
-            comp = parser.comp;
-        }
-        if (parser.slots) {
-            override(parser.slots, true);
-            parser.slots = nullptr;
-        }
-        builder->build(comp);
-
-        release();
-    }
-    rebuild = false;
+    if (comp) builder->update(comp, frameNo);      //update frame
+    else if (prepare()) builder->update(comp, 0);  //initial loading
+    build = false;
 }
 
 
 void LottieLoader::release()
 {
     if (copy) {
-        free((char*)content);
+        tvg::free((char*)content);
         content = nullptr;
     }
 }
@@ -68,7 +81,7 @@ void LottieLoader::release()
 /* External Class Implementation                                        */
 /************************************************************************/
 
-LottieLoader::LottieLoader() : FrameModule(FileType::Lottie), builder(new LottieBuilder)
+LottieLoader::LottieLoader() : FrameModule(FileType::Lot), builder(new LottieBuilder)
 {
 
 }
@@ -84,7 +97,7 @@ LottieLoader::~LottieLoader()
     delete(comp);
     delete(builder);
 
-    free(dirName);
+    tvg::free(dirName);
 }
 
 
@@ -93,17 +106,14 @@ bool LottieLoader::header()
     //A single thread doesn't need to perform intensive tasks.
     if (TaskScheduler::threads() == 0) {
         LoadModule::read();
-        run(0);
-        if (comp) {
+        if (prepare()) {
             w = static_cast<float>(comp->w);
             h = static_cast<float>(comp->h);
-            frameDuration = comp->duration();
-            frameCnt = comp->frameCnt();
+            segmentEnd = frameCnt = comp->frameCnt();
             frameRate = comp->frameRate;
             return true;
-        } else {
-            return false;
         }
+        return false;
     }
 
     //Quickly validate the given Lottie file without parsing in order to get the animation info.
@@ -128,7 +138,7 @@ bool LottieLoader::header()
             ++p;
             continue;
         }
-        //version.
+        //version
         if (!strncmp(p, "\"v\":", 4)) {
             p += 4;
             continue;
@@ -139,7 +149,7 @@ bool LottieLoader::header()
             p += 5;
             auto e = strstr(p, ",");
             if (!e) e = strstr(p, "}");
-            frameRate = strToFloat(p, nullptr);
+            frameRate = toFloat(p, nullptr);
             p = e;
             continue;
         }
@@ -149,7 +159,7 @@ bool LottieLoader::header()
             p += 5;
             auto e = strstr(p, ",");
             if (!e) e = strstr(p, "}");
-            startFrame = strToFloat(p, nullptr);
+            startFrame = toFloat(p, nullptr);
             p = e;
             continue;
         }
@@ -159,7 +169,7 @@ bool LottieLoader::header()
             p += 5;
             auto e = strstr(p, ",");
             if (!e) e = strstr(p, "}");
-            endFrame = strToFloat(p, nullptr);
+            endFrame = toFloat(p, nullptr);
             p = e;
             continue;
         }
@@ -169,7 +179,7 @@ bool LottieLoader::header()
             p += 4;
             auto e = strstr(p, ",");
             if (!e) e = strstr(p, "}");
-            w = strToFloat(p, nullptr);
+            w = toFloat(p, nullptr);
             p = e;
             continue;
         }
@@ -178,7 +188,7 @@ bool LottieLoader::header()
             p += 4;
             auto e = strstr(p, ",");
             if (!e) e = strstr(p, "}");
-            h = strToFloat(p, nullptr);
+            h = toFloat(p, nullptr);
             p = e;
             continue;
         }
@@ -190,62 +200,43 @@ bool LottieLoader::header()
         return false;
     }
 
-    frameCnt = (endFrame - startFrame);
-    frameDuration = frameCnt / frameRate;
+    segmentEnd = frameCnt = (endFrame - startFrame);
 
-    TVGLOG("LOTTIE", "info: frame rate = %f, duration = %f size = %f x %f", frameRate, frameDuration, w, h);
+    TVGLOG("LOTTIE", "info: frame rate = %f, duration = %f size = %f x %f", frameRate, frameCnt / frameRate, w, h);
 
     return true;
 }
 
 
-bool LottieLoader::open(const char* data, uint32_t size, bool copy)
+bool LottieLoader::open(const char* data, uint32_t size, const char* rpath, bool copy)
 {
     if (copy) {
-        content = (char*)malloc(size + 1);
+        content = tvg::malloc<char>(size + 1);
         if (!content) return false;
         memcpy((char*)content, data, size);
         const_cast<char*>(content)[size] = '\0';
     } else content = data;
 
-    this->dirName = strdup(".");
-
     this->size = size;
     this->copy = copy;
 
+    if (!rpath) this->dirName = duplicate(".");
+    else this->dirName = duplicate(rpath);
+
     return header();
 }
 
 
-bool LottieLoader::open(const string& path)
+bool LottieLoader::open(const char* path)
 {
 #ifdef THORVG_FILE_IO_SUPPORT
-    auto f = fopen(path.c_str(), "r");
-    if (!f) return false;
-
-    fseek(f, 0, SEEK_END);
-
-    size = ftell(f);
-    if (size == 0) {
-        fclose(f);
-        return false;
+    if ((content = LoadModule::open(path, size, true))) {
+        dirName = tvg::dirname(path);
+        copy = true;
+        return header();
     }
-
-    auto content = (char*)(malloc(sizeof(char) * size + 1));
-    fseek(f, 0, SEEK_SET);
-    size = fread(content, sizeof(char), size, f);
-    content[size] = '\0';
-
-    fclose(f);
-
-    this->dirName = strDirname(path.c_str());
-    this->content = content;
-    this->copy = true;
-
-    return header();
-#else
-    return false;
 #endif
+    return false;
 }
 
 
@@ -259,9 +250,8 @@ bool LottieLoader::resize(Paint* paint, float w, float h)
     paint->transform(m);
 
     //apply the scale to the base clipper
-    const Paint* clipper;
-    paint->composite(&clipper);
-    if (clipper) const_cast<Paint*>(clipper)->transform(m);
+    auto clipper = PAINT(paint)->clipper;
+    if (clipper) clipper->transform(m);
 
     return true;
 }
@@ -282,7 +272,7 @@ bool LottieLoader::read()
 
 Paint* LottieLoader::paint()
 {
-    done();
+    sync();
 
     if (!comp) return nullptr;
     comp->initiated = true;
@@ -290,62 +280,118 @@ Paint* LottieLoader::paint()
 }
 
 
-bool LottieLoader::override(const char* slots, bool byDefault)
+bool LottieLoader::apply(uint32_t slotcode, bool byDefault)
 {
+    if (curSlot == slotcode) return true;
+
     if (!ready() || comp->slots.count == 0) return false;
 
-    //override slots
-    if (slots) {
-        //Copy the input data because the JSON parser will encode the data immediately.
-        auto temp = byDefault ? slots : strdup(slots);
+    auto applied = false;
 
-        //parsing slot json
-        LottieParser parser(temp, dirName);
-        parser.comp = comp;
-
-        auto idx = 0;
-        auto succeed = false;
-        while (auto sid = parser.sid(idx == 0)) {
-            auto applied = false;
-            for (auto s = comp->slots.begin(); s < comp->slots.end(); ++s) {
-                if (strcmp((*s)->sid, sid)) continue;
-                if (parser.apply(*s, byDefault)) succeed = applied = true;
-                break;
+    // Reset all slots if slotcode is 0
+    if (slotcode == 0) {
+        ARRAY_FOREACH(p, comp->slots) (*p)->reset();
+        applied = true;
+    } else {
+        //Find the custom slot with the slotcode
+        INLIST_FOREACH(this->slots, slot) {
+            if (slot->code != slotcode) continue;
+            //apply the custom slot property to the targets.
+            ARRAY_FOREACH(p, slot->props) {
+                p->target->apply(p->prop, byDefault);
             }
-            if (!applied) parser.skip(sid);
-            ++idx;
+            applied = true;
+            break;
         }
-        free((char*)temp);
-        rebuild = succeed;
-        overridden |= succeed;
-        return rebuild;
-    //reset slots
-    } else if (overridden) {
-        for (auto s = comp->slots.begin(); s < comp->slots.end(); ++s) {
-            (*s)->reset();
+    }
+    curSlot = slotcode;
+    if (applied) build = true;
+    return applied;
+}
+
+
+bool LottieLoader::del(uint32_t slotcode, bool byDefault)
+{
+    if (comp->slots.empty() || slotcode == 0 || !ready()) return false;
+
+    // Search matching value and remove
+    INLIST_SAFE_FOREACH(this->slots, slot) {
+        if (slot->code != slotcode) continue;
+        if (!byDefault) {
+            ARRAY_FOREACH(p, slot->props) {
+                p->target->reset();
+            }
+            build = true;
         }
-        overridden = false;
-        rebuild = true;
+        this->slots.remove(slot);
+        delete(slot);
+        break;
     }
     return true;
 }
 
 
+uint32_t LottieLoader::gen(const char* slots, bool byDefault)
+{
+    if (!slots || !ready() || comp->slots.empty()) return 0;
+
+    //parsing slot json
+    auto temp = byDefault ? slots : duplicate(slots);
+    LottieParser parser(temp, dirName, builder->expressions());
+    parser.comp = comp;
+    
+    auto idx = 0;
+    auto custom = new LottieCustomSlot(djb2Encode(slots));
+
+    //Generates list of the custom slot overriding
+    while (auto sid = djb2Encode(parser.sid(idx == 0))) {
+        //Associates the overrding target to apply for the current custom slot
+        auto found = false;
+        ARRAY_FOREACH(p, comp->slots) {
+            if ((*p)->sid != sid) continue;  //find target
+            if (auto prop = parser.parse(*p)) custom->props.push({prop, *p});
+            found = true;
+            break;
+        }
+
+        if (!found) parser.skip(); //skip the value if the target slot is not found
+        ++idx;
+    }
+
+    tvg::free((char*)temp);
+
+    //Success, valid custom slot.
+    if (custom->props.count > 0) {
+        this->slots.back(custom);
+        return custom->code;
+    }
+
+    delete(custom);
+    return 0;
+}
+
+
+float LottieLoader::shorten(float frameNo)
+{
+    //This ensures that the target frame number is reached.
+    return nearbyintf((frameNo + startFrame()) * 10000.0f) * 0.0001f;
+}
+
+
 bool LottieLoader::frame(float no)
 {
-    auto frameNo = no + startFrame();
-
-    //This ensures that the target frame number is reached.
-    frameNo *= 10000.0f;
-    frameNo = nearbyintf(frameNo);
-    frameNo *= 0.0001f;
+    no = shorten(no);
 
     //Skip update if frame diff is too small.
-    if (fabsf(this->frameNo - frameNo) <= 0.0009f) return false;
+    if (!builder->tweening() && fabsf(this->frameNo - no) <= 0.0009f) return false;
 
     this->done();
 
-    this->frameNo = frameNo;
+    this->frameNo = no;
+
+    builder->offTween();
+
+    if (comp) comp->clear();     //clear synchronously
 
     TaskScheduler::request(this);
 
@@ -355,13 +401,13 @@ bool LottieLoader::frame(float no)
 
 float LottieLoader::startFrame()
 {
-    return frameCnt * segmentBegin;
+    return segmentBegin;
 }
 
 
 float LottieLoader::totalFrame()
 {
-    return (segmentEnd - segmentBegin) * frameCnt;
+    return segmentEnd - segmentBegin;
 }
 
 
@@ -373,8 +419,7 @@ float LottieLoader::curFrame()
 
 float LottieLoader::duration()
 {
-    if (segmentBegin == 0.0f && segmentEnd == 1.0f) return frameDuration;
-    return frameCnt * (segmentEnd - segmentBegin) / frameRate;
+    return (segmentEnd - segmentBegin) / frameRate;
 }
 
 
@@ -382,7 +427,7 @@ void LottieLoader::sync()
 {
     done();
 
-    if (rebuild) run(0);
+    if (build) run(0);
 }
 
 
@@ -400,14 +445,28 @@ const char* LottieLoader::markers(uint32_t index)
 }
 
 
+Result LottieLoader::segment(float begin, float end)
+{
+    if (begin < 0.0f) begin = 0.0f;
+    if (end > frameCnt) end = frameCnt;
+
+    if (begin > end) return Result::InvalidArguments;
+
+    segmentBegin = begin;
+    segmentEnd = end;
+
+    return Result::Success;
+}
+
+
 bool LottieLoader::segment(const char* marker, float& begin, float& end)
 {
     if (!ready() || comp->markers.count == 0) return false;
 
-    for (auto m = comp->markers.begin(); m < comp->markers.end(); ++m) {
-        if (!strcmp(marker, (*m)->name)) {
-            begin = (*m)->time / frameCnt;
-            end = ((*m)->time + (*m)->duration) / frameCnt;
+    ARRAY_FOREACH(p, comp->markers) {
+        if (!strcmp(marker, (*p)->name)) {
+            begin = (*p)->time;
+            end = (*p)->time + (*p)->duration;
             return true;
         }
     }
@@ -424,4 +483,50 @@ bool LottieLoader::ready()
     done();
     if (comp) return true;
     return false;
-}
\ No newline at end of file
+}
+
+
+bool LottieLoader::tween(float from, float to, float progress)
+{
+    //tweening is not necessary
+    if (tvg::zero(progress)) return frame(from);
+    else if (tvg::equal(progress, 1.0f)) return frame(to);
+
+    done();
+
+    frameNo = shorten(from);
+
+    builder->onTween(shorten(to), progress);
+
+    if (comp) comp->clear();     //clear synchronously
+
+    TaskScheduler::request(this);
+
+    return true;
+}
+
+
+bool LottieLoader::assign(const char* layer, uint32_t ix, const char* var, float val)
+{
+    if (!ready() || !comp->expressions) return false;
+    comp->root->assign(layer, ix, var, val);
+
+    return true;
+}
+
+
+bool LottieLoader::quality(uint8_t value)
+{
+    if (!ready()) return false;
+    if (comp->quality != value) {
+        comp->quality = value;
+        build = true;
+    }
+    return true;
+}
+
+
+void LottieLoader::set(const AssetResolver* resolver)
+{
+    builder->resolver = resolver;
+}
