@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2023 - 2024 the ThorVG project. All rights reserved.
+ * Copyright (c) 2023 - 2026 ThorVG project. All rights reserved.
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,9 +23,10 @@
 #ifndef _TVG_LOTTIE_MODEL_H_
 #define _TVG_LOTTIE_MODEL_H_
 
-#include <cstring>
-
 #include "tvgCommon.h"
+#include "tvgStr.h"
+#include "tvgCompressor.h"
+#include "tvgInlist.h"
 #include "tvgRender.h"
 #include "tvgLottieProperty.h"
 #include "tvgLottieRenderPooler.h"
@@ -83,13 +84,74 @@ struct LottieStroke
 
 struct LottieEffect
 {
-    enum Type : uint8_t {Tint = 20, Fill, Stroke, Tritone, DropShadow = 25, GaussianBlur = 29};
+    enum Type : uint8_t {Custom = 5, Tint = 20, Fill, Stroke, Tritone, DropShadow = 25, GaussianBlur = 29};
 
     virtual ~LottieEffect() {}
+
+    unsigned long nm;  //encoded by djb2
+    unsigned long mn;  //encoded by djb2
+    int16_t ix;
     Type type;
     bool enable = false;
 };
 
+
+struct LottieFxCustom : LottieEffect
+{
+    struct Property
+    {
+        LottieProperty* property;
+        unsigned long nm = 0;  //encoded by djb2
+        unsigned long mn = 0;  //encoded by djb2
+    };
+
+    char* name = nullptr;
+    Array<Property> props;
+
+    LottieFxCustom()
+    {
+        type = LottieEffect::Custom;
+    }
+
+    ~LottieFxCustom()
+    {
+        ARRAY_FOREACH(p, props) delete(p->property);
+    }
+
+    Property* property(int type)
+    {
+        LottieProperty* prop = nullptr;
+
+        switch (type) {
+            case 0: //slider
+            case 1: prop = new LottieFloat; break; //angle
+            case 2: prop = new LottieColor; break; //color
+            case 3: prop = new LottieVector; break; //point
+            case 4: //checkbox
+            case 7: //dropdown
+            case 10: prop = new LottieInteger; break;  //effect layer
+            case 6: TVGLOG("LOTTIE", "custom ignored?"); break;
+            default:
+                TVGLOG("LOTTIE", "missing custom property = %d\n", type);
+                return nullptr;
+        }
+        if (prop) {
+            props.push({prop, });
+            return &props.last();
+        }
+        return nullptr;
+    }
+
+    LottieProperty* property(const char* name)
+    {
+        auto id = djb2Encode(name);
+        ARRAY_FOREACH(p, props) {
+            if (p->mn == id || p->nm == id) return p->property;
+        }
+        return nullptr;
+    }
+};
+
 struct LottieFxFill : LottieEffect
 {
     //LottieInteger mask;
@@ -143,6 +205,7 @@ struct LottieFxTritone : LottieEffect
     LottieColor bright;
     LottieColor midtone;
     LottieColor dark;
+    LottieOpacity blend;
 
     LottieFxTritone()
     {
@@ -182,7 +245,7 @@ struct LottieMask
     LottiePathSet pathset;
     LottieFloat expand = 0.0f;
     LottieOpacity opacity = 255;
-    CompositeMethod method;
+    MaskMethod method;
     bool inverse = false;
 };
 
@@ -208,22 +271,24 @@ struct LottieObject
         Text,
         Repeater,
         RoundedCorner,
-        OffsetPath
+        OffsetPath,
+        TextRange
     };
 
     virtual ~LottieObject()
     {
     }
 
-    virtual void override(LottieProperty* prop, bool shallow, bool release)
+    virtual LottieProperty* override(LottieProperty* prop, bool release)
     {
         TVGERR("LOTTIE", "Unsupported slot type");
+        return nullptr;
     }
 
     virtual bool mergeable() { return false; }
     virtual LottieProperty* property(uint16_t ix) { return nullptr; }
 
-    unsigned long id = 0;
+    unsigned long id = 0;      //unique id by name generated by djb2 encoding
     Type type;
     bool hidden = false;       //remove?
 };
@@ -246,13 +311,13 @@ struct LottieGlyph
 
     ~LottieGlyph()
     {
-        for (auto p = children.begin(); p < children.end(); ++p) delete(*p);
-        free(code);
+        ARRAY_FOREACH(p, children) delete(*p);
+        tvg::free(code);
     }
 };
 
 
-struct LottieTextRange
+struct LottieTextRange : LottieObject
 {
     enum Based : uint8_t { Chars = 1, CharsExcludingSpaces, Words, Lines };
     enum Shape : uint8_t { Square = 1, RampUp, RampDown, Triangle, Round, Smooth };
@@ -260,6 +325,7 @@ struct LottieTextRange
 
     LottieTextRange()
     {
+        LottieObject::type = LottieObject::TextRange;
         style.flags.fillColor = 0;
         style.flags.strokeColor = 0;
         style.flags.strokeWidth = 0;
@@ -267,16 +333,16 @@ struct LottieTextRange
 
     ~LottieTextRange()
     {
-        free(interpolator);
+        tvg::free(interpolator);
     }
 
     struct {
-        LottieColor fillColor = RGB24{255, 255, 255};
-        LottieColor strokeColor = RGB24{255, 255, 255};
-        LottiePosition position = Point{0, 0};
-        LottiePoint scale = Point{100, 100};
-        LottieFloat letterSpacing = 0.0f;
-        LottieFloat lineSpacing = 0.0f;
+        LottieColor fillColor = RGB32{255, 255, 255};
+        LottieColor strokeColor = RGB32{255, 255, 255};
+        LottieVector position = Point{0, 0};
+        LottieScalar scale = Point{100, 100};
+        LottieFloat letterSpace = 0.0f;
+        LottieFloat lineSpace = 0.0f;
         LottieFloat strokeWidth = 0.0f;
         LottieFloat rotation = 0.0f;
         LottieOpacity fillOpacity = 255;
@@ -305,21 +371,72 @@ struct LottieTextRange
 
     float factor(float frameNo, float totalLen, float idx);
 
-    void color(float frameNo, RGB24& fillColor, RGB24& strokeColor, float factor, LottieExpressions* exps)
+    void color(float frameNo, RGB32& fillColor, RGB32& strokeColor, float factor, Tween& tween, LottieExpressions* exps)
     {
         if (style.flags.fillColor) {
-            auto color = style.fillColor(frameNo, exps);
-            fillColor.rgb[0] = tvg::lerp<uint8_t>(fillColor.rgb[0], color.rgb[0], factor);
-            fillColor.rgb[1] = tvg::lerp<uint8_t>(fillColor.rgb[1], color.rgb[1], factor);
-            fillColor.rgb[2] = tvg::lerp<uint8_t>(fillColor.rgb[2], color.rgb[2], factor);
+            auto color = style.fillColor(frameNo, tween, exps);
+            fillColor.r = tvg::lerp<uint8_t>(fillColor.r, color.r, factor);
+            fillColor.g = tvg::lerp<uint8_t>(fillColor.g, color.g, factor);
+            fillColor.b = tvg::lerp<uint8_t>(fillColor.b, color.b, factor);
         }
         if (style.flags.strokeColor) {
-            auto color = style.strokeColor(frameNo, exps);
-            strokeColor.rgb[0] = tvg::lerp<uint8_t>(strokeColor.rgb[0], color.rgb[0], factor);
-            strokeColor.rgb[1] = tvg::lerp<uint8_t>(strokeColor.rgb[1], color.rgb[1], factor);
-            strokeColor.rgb[2] = tvg::lerp<uint8_t>(strokeColor.rgb[2], color.rgb[2], factor);
+            auto color = style.strokeColor(frameNo, tween, exps);
+            strokeColor.r = tvg::lerp<uint8_t>(strokeColor.r, color.r, factor);
+            strokeColor.g = tvg::lerp<uint8_t>(strokeColor.g, color.g, factor);
+            strokeColor.b = tvg::lerp<uint8_t>(strokeColor.b, color.b, factor);
         }
     }
+
+    LottieProperty* override(LottieProperty* prop, bool release) override
+    {
+        LottieProperty* backup = nullptr;
+        if (style.fillColor.sid == prop->sid) {
+            if (release) style.fillColor.release();
+            else backup = new LottieColor(style.fillColor);
+            style.fillColor.copy(*static_cast<LottieColor*>(prop), false);
+        } else if (style.strokeColor.sid == prop->sid) {
+            if (release) style.strokeColor.release();
+            else backup = new LottieColor(style.strokeColor);
+            style.strokeColor.copy(*static_cast<LottieColor*>(prop), false);
+        } else if (style.position.sid == prop->sid) {
+            if (release) style.position.release();
+            else backup = new LottieVector(style.position);
+            style.position.copy(*static_cast<LottieVector*>(prop), false);
+        } else if (style.scale.sid == prop->sid) {
+            if (release) style.scale.release();
+            else backup = new LottieScalar(style.scale);
+            style.scale.copy(*static_cast<LottieScalar*>(prop), false);
+        } else if (style.rotation.sid == prop->sid) {
+            if (release) style.rotation.release();
+            else backup = new LottieFloat(style.rotation);
+            style.rotation.copy(*static_cast<LottieFloat*>(prop), false);
+        } else if (style.letterSpace.sid == prop->sid) {
+            if (release) style.letterSpace.release();
+            else backup = new LottieFloat(style.letterSpace);
+            style.letterSpace.copy(*static_cast<LottieFloat*>(prop), false);
+        } else if (style.lineSpace.sid == prop->sid) {
+            if (release) style.lineSpace.release();
+            else backup = new LottieFloat(style.lineSpace);
+            style.lineSpace.copy(*static_cast<LottieFloat*>(prop), false);
+        } else if (style.strokeWidth.sid == prop->sid) {
+            if (release) style.strokeWidth.release();
+            else backup = new LottieFloat(style.strokeWidth);
+            style.strokeWidth.copy(*static_cast<LottieFloat*>(prop), false);
+        } else if (style.fillOpacity.sid == prop->sid) {
+            if (release) style.fillOpacity.release();
+            else backup = new LottieOpacity(style.fillOpacity);
+            style.fillOpacity.copy(*static_cast<LottieOpacity*>(prop), false);
+        } else if (style.strokeOpacity.sid == prop->sid) {
+            if (release) style.strokeOpacity.release();
+            else backup = new LottieOpacity(style.strokeOpacity);
+            style.strokeOpacity.copy(*static_cast<LottieOpacity*>(prop), false);
+        } else if (style.opacity.sid == prop->sid) {
+            if (release) style.opacity.release();
+            else backup = new LottieOpacity(style.opacity);
+            style.opacity.copy(*static_cast<LottieOpacity*>(prop), false);
+        }
+        return backup;
+    }
 };
 
 
@@ -329,23 +446,23 @@ struct LottieFont
 
     ~LottieFont()
     {
-        for (auto c = chars.begin(); c < chars.end(); ++c) delete(*c);
-        free(style);
-        free(family);
-        free(name);
-        free(data.b64src);
+        ARRAY_FOREACH(p, chars) delete(*p);
+        tvg::free(style);
+        tvg::free(family);
+        tvg::free(name);
+        tvg::free(b64src);
     }
 
-    struct {
+    union {
         char* b64src = nullptr;
-        uint32_t size = 0;
-    } data;
+        char* path;
+    };
 
     Array<LottieGlyph*> chars;
     char* name = nullptr;
     char* family = nullptr;
     char* style = nullptr;
-    size_t dataSize = 0;
+    uint32_t size = 0;
     float ascent = 0.0f;
     Origin origin = Local;
 
@@ -360,28 +477,54 @@ struct LottieMarker
     
     ~LottieMarker()
     {
-        free(name);
+        tvg::free(name);
     }
 };
 
+
+struct LottieTextFollowPath
+{
+private:
+    RenderPath path;
+    PathCommand* cmds;
+    uint32_t cmdsCnt;
+    Point* pts;
+    Point* start;
+    float totalLen;
+    float currentLen;
+    Point split(float dLen, float lenSearched, float& angle);
+
+public:
+    LottieFloat firstMargin = 0.0f;
+    LottieMask* mask;
+    int8_t maskIdx = -1;
+
+    Point position(float lenSearched, float& angle);
+    float prepare(LottieMask* mask, float frameNo, float scale, Tween& tween, LottieExpressions* exps);
+};
+
+
 struct LottieText : LottieObject, LottieRenderPooler<tvg::Shape>
 {
     struct AlignOption
     {
         enum Group : uint8_t { Chars = 1, Word = 2, Line = 3, All = 4 };
-        Group grouping = Chars;
-        LottiePoint anchor{};
-    } alignOption;
+        Group group = Chars;
+        LottieScalar anchor{};
+    } alignOp;
 
     LottieText()
     {
         LottieObject::type = LottieObject::Text;
     }
 
-    void override(LottieProperty* prop, bool shallow, bool byDefault = false) override
+    LottieProperty* override(LottieProperty* prop, bool release) override
     {
-        if (byDefault) doc.release();
-        doc.copy(*static_cast<LottieTextDoc*>(prop), shallow);
+        LottieProperty* backup = nullptr;
+        if (release) doc.release();
+        else backup = new LottieTextDoc(doc);
+        doc.copy(*static_cast<LottieTextDoc*>(prop), false);
+        return backup;
     }
 
     LottieProperty* property(uint16_t ix) override
@@ -392,11 +535,13 @@ struct LottieText : LottieObject, LottieRenderPooler<tvg::Shape>
 
     LottieTextDoc doc;
     LottieFont* font = nullptr;
+    LottieTextFollowPath* follow = nullptr;
     Array<LottieTextRange*> ranges;
 
     ~LottieText()
     {
-        for (auto r = ranges.begin(); r < ranges.end(); ++r) delete(*r);
+        ARRAY_FOREACH(p, ranges) delete(*p);
+        delete(follow);
     }
 };
 
@@ -424,7 +569,7 @@ struct LottieTrimpath : LottieObject
         return nullptr;
     }
 
-    void segment(float frameNo, float& start, float& end, LottieExpressions* exps);
+    void segment(float frameNo, float& start, float& end, Tween& tween, LottieExpressions* exps);
 
     LottieFloat start = 0.0f;
     LottieFloat end = 100.0f;
@@ -433,7 +578,7 @@ struct LottieTrimpath : LottieObject
 };
 
 
-struct LottieShape : LottieObject, LottieRenderPooler<tvg::Shape>
+struct LottieShape : LottieObject, LottieRenderPooler<Shape>
 {
     bool clockwise = true;   //clockwise or counter-clockwise
 
@@ -494,8 +639,8 @@ struct LottieRect : LottieShape
         return nullptr;
     }
 
-    LottiePosition position = Point{0.0f, 0.0f};
-    LottiePoint size = Point{0.0f, 0.0f};
+    LottieVector position = Point{0.0f, 0.0f};
+    LottieScalar size = Point{0.0f, 0.0f};
     LottieFloat radius = 0.0f;       //rounded corner radius
 };
 
@@ -518,7 +663,7 @@ struct LottiePolyStar : LottieShape
         return nullptr;
     }
 
-    LottiePosition position = Point{0.0f, 0.0f};
+    LottieVector position = Point{0.0f, 0.0f};
     LottieFloat innerRadius = 0.0f;
     LottieFloat outerRadius = 0.0f;
     LottieFloat innerRoundness = 0.0f;
@@ -540,8 +685,8 @@ struct LottieEllipse : LottieShape
         return nullptr;
     }
 
-    LottiePosition position = Point{0.0f, 0.0f};
-    LottiePoint size = Point{0.0f, 0.0f};
+    LottieVector position = Point{0.0f, 0.0f};
+    LottieScalar size = Point{0.0f, 0.0f};
 };
 
 
@@ -597,37 +742,41 @@ struct LottieTransform : LottieObject
         return nullptr;
     }
 
-    void override(LottieProperty* prop, bool shallow, bool release) override
+    LottieProperty* override(LottieProperty* prop, bool release) override
     {
-        switch (prop->type) {
-            case LottieProperty::Type::Position: {
-                if (release) position.release();
-                position.copy(*static_cast<LottiePosition*>(prop), shallow);
-                break;
-            }
-            case LottieProperty::Type::Float: {
-                if (release) rotation.release();
-                rotation.copy(*static_cast<LottieFloat*>(prop), shallow);
-                break;
-            }
-            case LottieProperty::Type::Point: {
-                if (release) scale.release();
-                scale.copy(*static_cast<LottiePoint*>(prop), shallow);
-                break;
-            }
-            case LottieProperty::Type::Opacity: {
-                if (release) opacity.release();
-                opacity.copy(*static_cast<LottieOpacity*>(prop), shallow);
-                break;
-            }
-            default: break;
+        LottieProperty* backup = nullptr;
+        if (rotation.sid == prop->sid) {
+            if (release) rotation.release();
+            else backup = new LottieFloat(rotation);
+            rotation.copy(*static_cast<LottieFloat*>(prop), false);
+        } else if (scale.sid == prop->sid) {
+            if (release) scale.release();
+            else backup = new LottieScalar(scale);
+            scale.copy(*static_cast<LottieScalar*>(prop), false);
+        } else if (position.sid == prop->sid) {
+            if (release) position.release();
+            else backup = new LottieVector(position);
+            position.copy(*static_cast<LottieVector*>(prop), false);
+        } else if (opacity.sid == prop->sid) {
+            if (release) opacity.release();
+            else backup = new LottieOpacity(opacity);
+            opacity.copy(*static_cast<LottieOpacity*>(prop), false);
+        } else if (skewAngle.sid == prop->sid) {
+            if (release) skewAngle.release();
+            else backup = new LottieFloat(skewAngle);
+            skewAngle.copy(*static_cast<LottieFloat*>(prop), false);
+        } else if (skewAxis.sid == prop->sid) {
+            if (release) skewAxis.release();
+            else backup = new LottieFloat(skewAxis);
+            skewAxis.copy(*static_cast<LottieFloat*>(prop), false);
         }
+        return backup;
     }
 
-    LottiePosition position = Point{0.0f, 0.0f};
+    LottieVector position = Point{0.0f, 0.0f};
     LottieFloat rotation = 0.0f;           //z rotation
-    LottiePoint scale = Point{100.0f, 100.0f};
-    LottiePoint anchor = Point{0.0f, 0.0f};
+    LottieScalar scale = Point{100.0f, 100.0f};
+    LottieScalar anchor = Point{0.0f, 0.0f};
     LottieOpacity opacity = 255;
     LottieFloat skewAngle = 0.0f;
     LottieFloat skewAxis = 0.0f;
@@ -639,7 +788,7 @@ struct LottieTransform : LottieObject
 
 struct LottieSolid : LottieObject 
 {
-    LottieColor color = RGB24{255, 255, 255};
+    LottieColor color = RGB32{255, 255, 255};
     LottieOpacity opacity = 255;
 
     LottieProperty* property(uint16_t ix) override
@@ -668,10 +817,13 @@ struct LottieSolidStroke : LottieSolid, LottieStroke
         return LottieSolid::property(ix);
     }
 
-    void override(LottieProperty* prop, bool shallow, bool release) override
+    LottieProperty* override(LottieProperty* prop, bool release) override
     {
+        LottieProperty* backup = nullptr;
         if (release) color.release();
-        color.copy(*static_cast<LottieColor*>(prop), shallow);
+        else backup = new LottieColor(color);
+        color.copy(*static_cast<LottieColor*>(prop), false);
+        return backup;
     }
 };
 
@@ -683,18 +835,22 @@ struct LottieSolidFill : LottieSolid
         LottieObject::type = LottieObject::SolidFill;
     }
 
-    void override(LottieProperty* prop, bool shallow, bool release) override
+    LottieProperty* override(LottieProperty* prop, bool release) override
     {
-        if (prop->type == LottieProperty::Type::Opacity) {
-            if (release) opacity.release();
-            opacity.copy(*static_cast<LottieOpacity*>(prop), shallow);
-        } else if (prop->type == LottieProperty::Type::Color) {
+        LottieProperty* backup = nullptr;
+        if (color.sid == prop->sid) {
             if (release) color.release();
-            color.copy(*static_cast<LottieColor*>(prop), shallow);
+            else backup = new LottieColor(color);
+            color.copy(*static_cast<LottieColor*>(prop), false);
+        } else if (opacity.sid == prop->sid) {
+            if (release) opacity.release();
+            else backup = new LottieOpacity(opacity);
+            opacity.copy(*static_cast<LottieOpacity*>(prop), false);
         }
+        return backup;
     }
 
-    FillRule rule = FillRule::Winding;
+    FillRule rule = FillRule::NonZero;
 };
 
 
@@ -705,7 +861,7 @@ struct LottieGradient : LottieObject
         if (!colorStops.populated) {
             auto count = colorStops.count;  //colorstop count can be modified after population
             if (colorStops.frames) {
-                for (auto v = colorStops.frames->begin(); v < colorStops.frames->end(); ++v) {
+                ARRAY_FOREACH(v, *colorStops.frames) {
                     colorStops.count = populate(v->value, count);
                 }
             } else {
@@ -728,23 +884,27 @@ struct LottieGradient : LottieObject
         return nullptr;
     }
 
-    void override(LottieProperty* prop, bool shallow, bool release = false) override
+    LottieProperty* override(LottieProperty* prop, bool release) override
     {
+        LottieProperty* backup = nullptr;
         if (release) colorStops.release();
-        colorStops.copy(*static_cast<LottieColorStop*>(prop), shallow);
+        else backup = new LottieColorStop(colorStops);
+        colorStops.copy(*static_cast<LottieColorStop*>(prop), false);
         prepare();
+        return backup;
     }
 
     uint32_t populate(ColorStop& color, size_t count);
-    Fill* fill(float frameNo, LottieExpressions* exps);
+    Fill* fill(float frameNo, uint8_t opacity, Tween& tween, LottieExpressions* exps);
 
-    LottiePoint start = Point{0.0f, 0.0f};
-    LottiePoint end = Point{0.0f, 0.0f};
+    LottieScalar start = Point{0.0f, 0.0f};
+    LottieScalar end = Point{0.0f, 0.0f};
     LottieFloat height = 0.0f;
     LottieFloat angle = 0.0f;
     LottieOpacity opacity = 255;
     LottieColorStop colorStops;
-    uint8_t id = 0;    //1: linear, 2: radial
+    uint8_t id = 0; //1: linear, 2: radial
+    bool opaque = true; //fully opaque or not
 };
 
 
@@ -755,7 +915,7 @@ struct LottieGradientFill : LottieGradient
         LottieObject::type = LottieObject::GradientFill;
     }
 
-    FillRule rule = FillRule::Winding;
+    FillRule rule = FillRule::NonZero;
 };
 
 
@@ -778,19 +938,21 @@ struct LottieGradientStroke : LottieGradient, LottieStroke
 };
 
 
-struct LottieImage : LottieObject, LottieRenderPooler<tvg::Picture>
+struct LottieImage : LottieObject
 {
-    LottieBitmap data;
+    LottieBitmap bitmap;
+    bool resolved = false;
 
-    void override(LottieProperty* prop, bool shallow, bool release = false) override
+    LottieProperty* override(LottieProperty* prop, bool release) override
     {
-        if (release) data.release();
-        data.copy(*static_cast<LottieBitmap*>(prop), shallow);
-        update();
+        LottieProperty* backup = nullptr;
+        if (release) bitmap.release();
+        else backup = new LottieBitmap(bitmap);
+        bitmap.copy(*static_cast<LottieBitmap*>(prop), false);
+        return backup;
     }
 
     void prepare();
-    void update();
 };
 
 
@@ -818,10 +980,10 @@ struct LottieRepeater : LottieObject
     LottieFloat offset = 0.0f;
 
     //Transform
-    LottiePosition position = Point{0.0f, 0.0f};
+    LottieVector position = Point{0.0f, 0.0f};
     LottieFloat rotation = 0.0f;
-    LottiePoint scale = Point{100.0f, 100.0f};
-    LottiePoint anchor = Point{0.0f, 0.0f};
+    LottieScalar scale = Point{100.0f, 100.0f};
+    LottieScalar anchor = Point{0.0f, 0.0f};
     LottieOpacity startOpacity = 255;
     LottieOpacity endOpacity = 255;
     bool inorder = true;        //true: higher,  false: lower
@@ -847,19 +1009,20 @@ struct LottieGroup : LottieObject, LottieRenderPooler<tvg::Shape>
 
     virtual ~LottieGroup()
     {
-        for (auto p = children.begin(); p < children.end(); ++p) delete(*p);
+        ARRAY_FOREACH(p, children) delete(*p);
     }
 
     void prepare(LottieObject::Type type = LottieObject::Group);
     bool mergeable() override { return allowMerge; }
+    LottieProperty* property(uint16_t ix) override;
 
     LottieObject* content(unsigned long id)
     {
         if (this->id == id) return this;
 
         //source has children, find recursively.
-        for (auto c = children.begin(); c < children.end(); ++c) {
-            auto child = *c;
+        ARRAY_FOREACH(p, children) {
+            auto child = *p;
             if (child->type == LottieObject::Type::Group || child->type == LottieObject::Type::Layer) {
                 if (auto ret = static_cast<LottieGroup*>(child)->content(id)) return ret;
             } else if (child->id == id) return child;
@@ -869,6 +1032,7 @@ struct LottieGroup : LottieObject, LottieRenderPooler<tvg::Shape>
 
     Scene* scene = nullptr;
     Array<LottieObject*> children;
+    BlendMethod blendMethod = BlendMethod::Normal;
 
     bool reqFragment : 1;   //requirement to fragment the render context
     bool buildDone : 1;     //completed in building the composition.
@@ -884,17 +1048,11 @@ struct LottieLayer : LottieGroup
 
     ~LottieLayer();
 
-    uint8_t opacity(float frameNo)
-    {
-        //return zero if the visibility is false.
-        if (type == Null) return 255;
-        return transform->opacity(frameNo);
-    }
-
     bool mergeable() override { return false; }
-
-    void prepare(RGB24* color = nullptr);
+    void prepare(RGB32* color = nullptr);
     float remap(LottieComposition* comp, float frameNo, LottieExpressions* exp);
+    LottieProperty* property(uint16_t ix) override;
+    bool assign(const char* layer, uint32_t ix, const char* var, float val);
 
     char* name = nullptr;
     LottieLayer* parent = nullptr;
@@ -913,9 +1071,9 @@ struct LottieLayer : LottieGroup
     float outFrame = 0.0f;
     float startFrame = 0.0f;
     unsigned long rid = 0;      //pre-composition reference id.
-    int16_t mid = -1;           //id of the matte layer.
-    int16_t pidx = -1;          //index of the parent layer.
-    int16_t idx = -1;           //index of the current layer.
+    int16_t mix = -1;           //index of the matte layer.
+    int16_t pix = -1;           //index of the parent layer.
+    int16_t ix = -1;            //index of the current layer.
 
     struct {
         float frameNo = -1.0f;
@@ -923,28 +1081,44 @@ struct LottieLayer : LottieGroup
         uint8_t opacity;
     } cache;
 
-    CompositeMethod matteType = CompositeMethod::None;
-    BlendMethod blendMethod = BlendMethod::Normal;
+    MaskMethod matteType = MaskMethod::None;
     Type type = Null;
     bool autoOrient = false;
     bool matteSrc = false;
 
+    LottieEffect* effectById(unsigned long id)
+    {
+        ARRAY_FOREACH(p, effects) {
+            if (id == (*p)->nm || id == (*p)->mn) return *p;
+        }
+        return nullptr;
+    }
+
+    LottieEffect* effectByIdx(int16_t ix)
+    {
+        ARRAY_FOREACH(p, effects) {
+            if (ix == (*p)->ix) return *p;
+        }
+        return nullptr;
+    }
+
+
     LottieLayer* layerById(unsigned long id)
     {
-        for (auto child = children.begin(); child < children.end(); ++child) {
-            if ((*child)->type != LottieObject::Type::Layer) continue;
-            auto layer = static_cast<LottieLayer*>(*child);
+        ARRAY_FOREACH(p, children) {
+            if ((*p)->type != LottieObject::Type::Layer) continue;
+            auto layer = static_cast<LottieLayer*>(*p);
             if (layer->id == id) return layer;
         }
         return nullptr;
     }
 
-    LottieLayer* layerByIdx(int16_t idx)
+    LottieLayer* layerByIdx(int16_t ix)
     {
-        for (auto child = children.begin(); child < children.end(); ++child) {
-            if ((*child)->type != LottieObject::Type::Layer) continue;
-            auto layer = static_cast<LottieLayer*>(*child);
-            if (layer->idx == idx) return layer;
+        ARRAY_FOREACH(p, children) {
+            if ((*p)->type != LottieObject::Type::Layer) continue;
+            auto layer = static_cast<LottieLayer*>(*p);
+            if (layer->ix == ix) return layer;
         }
         return nullptr;
     }
@@ -953,26 +1127,25 @@ struct LottieLayer : LottieGroup
 
 struct LottieSlot
 {
-    struct Pair {
+    struct Pair
+    {
         LottieObject* obj;
         LottieProperty* prop;
     };
 
-    void assign(LottieObject* target, bool byDefault);
+    void add(uint32_t slotcode, LottieProperty* prop);
+    void apply(LottieProperty* prop, bool byDefault = false);
     void reset();
 
-    LottieSlot(LottieLayer* layer, LottieObject* parent, char* sid, LottieObject* obj, LottieProperty::Type type) : context{layer, parent}, sid(sid), type(type)
+    LottieSlot(LottieLayer* layer, LottieObject* parent, unsigned long sid, LottieObject* obj, LottieProperty::Type type) : context{layer, parent}, sid(sid), type(type)
     {
-        pairs.push({obj});
+        pairs.push({obj, nullptr});
     }
 
     ~LottieSlot()
     {
-        free(sid);
         if (!overridden) return;
-        for (auto pair = pairs.begin(); pair < pairs.end(); ++pair) {
-            delete(pair->prop);
-        }
+        ARRAY_FOREACH(pair, pairs) delete(pair->prop);
     }
 
     struct {
@@ -980,8 +1153,8 @@ struct LottieSlot
         LottieObject* parent;
     } context;
 
-    char* sid;
-    Array<Pair> pairs;
+    unsigned long sid;    // djb2 encoded
+    Array<Pair> pairs;    // object-property pairs that can be overridden by this slot
     LottieProperty::Type type;
 
     bool overridden = false;
@@ -992,6 +1165,11 @@ struct LottieComposition
 {
     ~LottieComposition();
 
+    void clear()
+    {
+        if (root && root->scene) root->scene->remove();
+    }
+
     float duration() const
     {
         return frameCnt() / frameRate;  // in second
@@ -1016,13 +1194,20 @@ struct LottieComposition
 
     LottieLayer* asset(unsigned long id)
     {
-        for (auto asset = assets.begin(); asset < assets.end(); ++asset) {
-            auto layer = static_cast<LottieLayer*>(*asset);
+        ARRAY_FOREACH(p, assets) {
+            auto layer = static_cast<LottieLayer*>(*p);
             if (layer->id == id) return layer;
         }
         return nullptr;
     }
 
+    void clamp(float& frameNo)
+    {
+        frameNo += root->inFrame;
+        if (frameNo < root->inFrame) frameNo = root->inFrame;
+        if (frameNo >= root->outFrame) frameNo = root->outFrame - 1;
+    }
+
     LottieLayer* root = nullptr;
     char* version = nullptr;
     char* name = nullptr;
@@ -1035,6 +1220,7 @@ struct LottieComposition
     Array<LottieMarker*> markers;
     bool expressions = false;
     bool initiated = false;
+    uint8_t quality = 50;
 };
 
 #endif //_TVG_LOTTIE_MODEL_H_
