@@ -14,12 +14,10 @@
 //
 // Author: Skal (pascal.massimino@gmail.com)
 
+#include "tvgCommon.h"
 #include "./quant_levels_dec.h"
 #include "./utils.h"
 
-#include <stdlib.h>
-#include <string.h>   // for memset
-
 // #define USE_DITHERING   // uncomment to enable ordered dithering (not vital)
 
 #define FIX 16     // fix-point precision for averaging
@@ -70,9 +68,8 @@ typedef struct {
 
 //------------------------------------------------------------------------------
 
-#define CLIP_MASK (int)(~0U << (8 + DFIX))
 static WEBP_INLINE uint8_t clip_8b(int v) {
-  return (!(v & CLIP_MASK)) ? (uint8_t)(v >> DFIX) : (v < 0) ? 0u : 255u;
+  return (!(v & (int)(~0U << (8 + DFIX)))) ? (uint8_t)(v >> DFIX) : (v < 0) ? 0u : 255u;
 }
 
 // vertical accumulation
@@ -216,7 +213,7 @@ static int InitParams(uint8_t* const data, int width, int height,
   const size_t size_m =  width * sizeof(*p->average_);
   const size_t size_lut = (1 + 2 * LUT_SIZE) * sizeof(*p->correction_);
   const size_t total_size = size_scratch_m + size_m + size_lut;
-  uint8_t* mem = (uint8_t*)malloc(1U * total_size);
+  uint8_t* mem = tvg::malloc<uint8_t>(1U * total_size);
 
   if (mem == NULL) return 0;
   p->mem_ = (void*)mem;
@@ -250,7 +247,7 @@ static int InitParams(uint8_t* const data, int width, int height,
 }
 
 static void CleanupParams(SmoothParams* const p) {
-  free(p->mem_);
+  tvg::free(p->mem_);
 }
 
 int WebPDequantizeLevels(uint8_t* const data, int width, int height,
