@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020 - 2024 the ThorVG project. All rights reserved.
+ * Copyright (c) 2020 - 2026 ThorVG project. All rights reserved.
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -28,14 +28,13 @@
 class GlShader
 {
 public:
-    static std::shared_ptr<GlShader> gen(const char* vertSrc, const char* fragSrc);
+    GlShader(const char* vertSrc, const char* fragSrc);
     ~GlShader();
 
     uint32_t getVertexShader();
     uint32_t getFragmentShader();
 
 private:
-    void    createShader(const char* vertSrc, const char* fragSrc);
     uint32_t compileShader(uint32_t type, char* shaderSrc);
 
     uint32_t mVtShader;
