@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020 - 2024 the ThorVG project. All rights reserved.
+ * Copyright (c) 2020 - 2026 ThorVG project. All rights reserved.
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -20,73 +20,59 @@
  * SOFTWARE.
  */
 
-#include <string.h>
-#include <math.h>
 #include "tvgSwCommon.h"
 
 /************************************************************************/
 /* Internal Class Implementation                                        */
 /************************************************************************/
 
-static constexpr auto SW_STROKE_TAG_POINT = 1;
-static constexpr auto SW_STROKE_TAG_CUBIC = 2;
-static constexpr auto SW_STROKE_TAG_BEGIN = 4;
-static constexpr auto SW_STROKE_TAG_END = 8;
+#define SW_STROKE_TAG_POINT 1
+#define SW_STROKE_TAG_CUBIC 2
+#define SW_STROKE_TAG_BEGIN 4
+#define SW_STROKE_TAG_END 8
 
-static inline SwFixed SIDE_TO_ROTATE(const int32_t s)
+static inline int64_t SIDE_TO_ROTATE(const int32_t s)
 {
-    return (SW_ANGLE_PI2 - static_cast<SwFixed>(s) * SW_ANGLE_PI);
+    return (SW_ANGLE_PI2 - static_cast<int64_t>(s) * SW_ANGLE_PI);
 }
 
 
 static inline void SCALE(const SwStroke& stroke, SwPoint& pt)
 {
-    pt.x = static_cast<SwCoord>(pt.x * stroke.sx);
-    pt.y = static_cast<SwCoord>(pt.y * stroke.sy);
+    pt.x = static_cast<int32_t>(pt.x * stroke.sx);
+    pt.y = static_cast<int32_t>(pt.y * stroke.sy);
 }
 
 
 static void _growBorder(SwStrokeBorder* border, uint32_t newPts)
 {
-    auto maxOld = border->maxPts;
-    auto maxNew = border->ptsCnt + newPts;
-
-    if (maxNew <= maxOld) return;
-
-    auto maxCur = maxOld;
-
-    while (maxCur < maxNew)
-        maxCur += (maxCur >> 1) + 16;
-    //OPTIMIZE: use mempool!
-    border->pts = static_cast<SwPoint*>(realloc(border->pts, maxCur * sizeof(SwPoint)));
-    border->tags = static_cast<uint8_t*>(realloc(border->tags, maxCur * sizeof(uint8_t)));
-    border->maxPts = maxCur;
+    if (border->pts.count + newPts <= border->pts.reserved) return;
+    border->pts.grow(newPts * 20);
+    border->tags = tvg::realloc<uint8_t>(border->tags, border->pts.reserved);      //align the pts / tags memory size
 }
 
 
 static void _borderClose(SwStrokeBorder* border, bool reverse)
 {
     auto start = border->start;
-    auto count = border->ptsCnt;
+    auto count = border->pts.count;
 
     //Don't record empty paths!
     if (count <= start + 1U) {
-        border->ptsCnt = start;
+        border->pts.count = start;
     } else {
         /* Copy the last point to the start of this sub-path,
            since it contains the adjusted starting coordinates */
-        border->ptsCnt = --count;
+        border->pts.count = --count;
         border->pts[start] = border->pts[count];
 
         if (reverse) {
             //reverse the points
-            auto pt1 = border->pts + start + 1;
-            auto pt2 = border->pts + count - 1;
+            auto pt1 = border->pts.data + start + 1;
+            auto pt2 = border->pts.data + count - 1;
 
             while (pt1 < pt2) {
-                auto tmp = *pt1;
-                *pt1 = *pt2;
-                *pt2 = tmp;
+                std::swap(*pt1, *pt2);
                 ++pt1;
                 --pt2;
             }
@@ -96,9 +82,7 @@ static void _borderClose(SwStrokeBorder* border, bool reverse)
             auto tag2 = border->tags + count - 1;
 
             while (tag1 < tag2) {
-                auto tmp = *tag1;
-                *tag1 = *tag2;
-                *tag2 = tmp;
+                std::swap(*tag1, *tag2);
                 ++tag1;
                 --tag2;
             }
@@ -117,26 +101,24 @@ static void _borderCubicTo(SwStrokeBorder* border, const SwPoint& ctrl1, const S
 {
     _growBorder(border, 3);
 
-    auto pt = border->pts + border->ptsCnt;
-    auto tag = border->tags + border->ptsCnt;
+    auto tag = border->tags + border->pts.count;
 
-    pt[0] = ctrl1;
-    pt[1] = ctrl2;
-    pt[2] = to;
+    border->pts.push(ctrl1);
+    border->pts.push(ctrl2);
+    border->pts.push(to);
 
     tag[0] = SW_STROKE_TAG_CUBIC;
     tag[1] = SW_STROKE_TAG_CUBIC;
     tag[2] = SW_STROKE_TAG_POINT;
 
-    border->ptsCnt += 3;
     border->movable = false;
 }
 
 
-static void _borderArcTo(SwStrokeBorder* border, const SwPoint& center, SwFixed radius, SwFixed angleStart, SwFixed angleDiff, SwStroke& stroke)
+static void _borderArcTo(SwStrokeBorder* border, const SwPoint& center, int64_t radius, int64_t angleStart, int64_t angleDiff, SwStroke& stroke)
 {
-    constexpr SwFixed ARC_CUBIC_ANGLE = SW_ANGLE_PI / 2;
-    SwPoint a = {static_cast<SwCoord>(radius), 0};
+    constexpr int64_t ARC_CUBIC_ANGLE = SW_ANGLE_PI / 2;
+    SwPoint a = {static_cast<int32_t>(radius), 0};
     mathRotate(a, angleStart);
     SCALE(stroke, a);
     a += center;
@@ -157,7 +139,7 @@ static void _borderArcTo(SwStrokeBorder* border, const SwPoint& center, SwFixed
         theta >>= 1;
 
         //compute end point
-        SwPoint b = {static_cast<SwCoord>(radius), 0};
+        SwPoint b = {static_cast<int32_t>(radius), 0};
         mathRotate(b, next);
         SCALE(stroke, b);
         b += center;
@@ -165,12 +147,12 @@ static void _borderArcTo(SwStrokeBorder* border, const SwPoint& center, SwFixed
         //compute first and second control points
         auto length = mathMulDiv(radius, mathSin(theta) * 4, (0x10000L + mathCos(theta)) * 3);
 
-        SwPoint a2 = {static_cast<SwCoord>(length), 0};
+        SwPoint a2 = {static_cast<int32_t>(length), 0};
         mathRotate(a2, angle + rotate);
         SCALE(stroke, a2);
         a2 += a;
 
-        SwPoint b2 = {static_cast<SwCoord>(length), 0};
+        SwPoint b2 = {static_cast<int32_t>(length), 0};
         mathRotate(b2, next - rotate);
         SCALE(stroke, b2);
         b2 += b;
@@ -190,15 +172,13 @@ static void _borderLineTo(SwStrokeBorder* border, const SwPoint& to, bool movabl
 {
     if (border->movable) {
         //move last point
-        border->pts[border->ptsCnt - 1] = to;
+        border->pts.last() = to;
     } else {
         //don't add zero-length line_to
-        if (border->ptsCnt > 0 && (border->pts[border->ptsCnt - 1] - to).small()) return;
-
+        if (!border->pts.empty() && (border->pts.last() - to).tiny()) return;
         _growBorder(border, 1);
-        border->pts[border->ptsCnt] = to;
-        border->tags[border->ptsCnt] = SW_STROKE_TAG_POINT;
-        border->ptsCnt += 1;
+        border->tags[border->pts.count] = SW_STROKE_TAG_POINT;
+        border->pts.push(to);
     }
 
     border->movable = movable;
@@ -210,7 +190,7 @@ static void _borderMoveTo(SwStrokeBorder* border, SwPoint& to)
     //close current open path if any?
     if (border->start >= 0) _borderClose(border, false);
 
-    border->start = border->ptsCnt;
+    border->start = border->pts.count;
     border->movable = false;
 
     _borderLineTo(border, to, false);
@@ -219,7 +199,7 @@ static void _borderMoveTo(SwStrokeBorder* border, SwPoint& to)
 
 static void _arcTo(SwStroke& stroke, int32_t side)
 {
-    auto border = stroke.borders + side;
+    auto border = stroke.borders[side];
     auto rotate = SIDE_TO_ROTATE(side);
     auto total = mathDiff(stroke.angleIn, stroke.angleOut);
     if (total == SW_ANGLE_PI) total = -rotate * 2;
@@ -229,18 +209,18 @@ static void _arcTo(SwStroke& stroke, int32_t side)
 }
 
 
-static void _outside(SwStroke& stroke, int32_t side, SwFixed lineLength)
+static void _outside(SwStroke& stroke, int32_t side, int64_t lineLength)
 {
-    auto border = stroke.borders + side;
+    auto border = stroke.borders[side];
 
     if (stroke.join == StrokeJoin::Round) {
         _arcTo(stroke, side);
     } else {
         //this is a mitered (pointed) or beveled (truncated) corner
         auto rotate = SIDE_TO_ROTATE(side);
-        auto bevel = (stroke.join == StrokeJoin::Bevel) ? true : false;
-        SwFixed phi = 0;
-        SwFixed thcos = 0;
+        auto bevel = stroke.join == StrokeJoin::Bevel;
+        int64_t phi = 0;
+        int64_t thcos = 0;
 
         if (!bevel) {
             auto theta = mathDiff(stroke.angleIn, stroke.angleOut);
@@ -261,7 +241,7 @@ static void _outside(SwStroke& stroke, int32_t side, SwFixed lineLength)
 
         //this is a bevel (broken angle)
         if (bevel) {
-            SwPoint delta = {static_cast<SwCoord>(stroke.width), 0};
+            SwPoint delta = {static_cast<int32_t>(stroke.width), 0};
             mathRotate(delta, stroke.angleOut + rotate);
             SCALE(stroke, delta);
             delta += stroke.center;
@@ -270,7 +250,7 @@ static void _outside(SwStroke& stroke, int32_t side, SwFixed lineLength)
         //this is a miter (intersection)
         } else {
             auto length = mathDivide(stroke.width, thcos);
-            SwPoint delta = {static_cast<SwCoord>(length), 0};
+            SwPoint delta = {static_cast<int32_t>(length), 0};
             mathRotate(delta, phi);
             SCALE(stroke, delta);
             delta += stroke.center;
@@ -279,7 +259,7 @@ static void _outside(SwStroke& stroke, int32_t side, SwFixed lineLength)
             /* Now add and end point
                Only needed if not lineto (lineLength is zero for curves) */
             if (lineLength == 0) {
-                delta = {static_cast<SwCoord>(stroke.width), 0};
+                delta = {static_cast<int32_t>(stroke.width), 0};
                 mathRotate(delta, stroke.angleOut + rotate);
                 SCALE(stroke, delta);
                 delta += stroke.center;
@@ -290,9 +270,9 @@ static void _outside(SwStroke& stroke, int32_t side, SwFixed lineLength)
 }
 
 
-static void _inside(SwStroke& stroke, int32_t side, SwFixed lineLength)
+static void _inside(SwStroke& stroke, int32_t side, int64_t lineLength)
 {
-    auto border = stroke.borders + side;
+    auto border = stroke.borders[side];
     auto theta = mathDiff(stroke.angleIn, stroke.angleOut) / 2;
     SwPoint delta;
     bool intersect = false;
@@ -301,14 +281,14 @@ static void _inside(SwStroke& stroke, int32_t side, SwFixed lineLength)
        lines are long enough (line length is zero for curves). */
     if (border->movable && lineLength > 0) {
         //compute minimum required length of lines
-        SwFixed minLength = abs(mathMultiply(stroke.width, mathTan(theta)));
+        int64_t minLength = abs(mathMultiply(stroke.width, mathTan(theta)));
         if (stroke.lineLength >= minLength && lineLength >= minLength) intersect = true;
     }
 
     auto rotate = SIDE_TO_ROTATE(side);
 
     if (!intersect) {
-        delta = {static_cast<SwCoord>(stroke.width), 0};
+        delta = {static_cast<int32_t>(stroke.width), 0};
         mathRotate(delta, stroke.angleOut + rotate);
         SCALE(stroke, delta);
         delta += stroke.center;
@@ -317,7 +297,7 @@ static void _inside(SwStroke& stroke, int32_t side, SwFixed lineLength)
         //compute median angle
         auto phi = stroke.angleIn + theta;
         auto thcos = mathCos(theta);
-        delta = {static_cast<SwCoord>(mathDivide(stroke.width, thcos)), 0};
+        delta = {static_cast<int32_t>(mathDivide(stroke.width, thcos)), 0};
         mathRotate(delta, phi + rotate);
         SCALE(stroke, delta);
         delta += stroke.center;
@@ -327,7 +307,7 @@ static void _inside(SwStroke& stroke, int32_t side, SwFixed lineLength)
 }
 
 
-void _processCorner(SwStroke& stroke, SwFixed lineLength)
+void _processCorner(SwStroke& stroke, int64_t lineLength)
 {
     auto turn = mathDiff(stroke.angleIn, stroke.angleOut);
 
@@ -348,19 +328,17 @@ void _processCorner(SwStroke& stroke, SwFixed lineLength)
 }
 
 
-void _firstSubPath(SwStroke& stroke, SwFixed startAngle, SwFixed lineLength)
+void _firstSubPath(SwStroke& stroke, int64_t startAngle, int64_t lineLength)
 {
-    SwPoint delta = {static_cast<SwCoord>(stroke.width), 0};
+    SwPoint delta = {static_cast<int32_t>(stroke.width), 0};
     mathRotate(delta, startAngle + SW_ANGLE_PI2);
     SCALE(stroke, delta);
 
     auto pt = stroke.center + delta;
-    auto border = stroke.borders;
-    _borderMoveTo(border, pt);
+    _borderMoveTo(stroke.borders[0], pt);
 
     pt = stroke.center - delta;
-    ++border;
-    _borderMoveTo(border, pt);
+    _borderMoveTo(stroke.borders[1], pt);
 
     /* Save angle, position and line length for last join
        lineLength is zero for curves */
@@ -374,19 +352,23 @@ static void _lineTo(SwStroke& stroke, const SwPoint& to)
 {
     auto delta = to - stroke.center;
 
-    //a zero-length lineto is a no-op; avoid creating a spurious corner
-    if (delta.zero()) return;
+    //a zero-length lineto is a no-op
+    if (delta.zero()) {
+        //round and square caps are expected to be drawn as a dot even for zero-length lines
+        if (stroke.firstPt && stroke.cap != StrokeCap::Butt) _firstSubPath(stroke, 0, 0); 
+        return; 
+    }
 
     /* The lineLength is used to determine the intersection of strokes outlines.
        The scale needs to be reverted since the stroke width has not been scaled.
        An alternative option is to scale the width of the stroke properly by
        calculating the mixture of the sx/sy rating on the stroke direction. */
-    delta.x = static_cast<SwCoord>(delta.x / stroke.sx);
-    delta.y = static_cast<SwCoord>(delta.y / stroke.sy);
+    delta.x = static_cast<int32_t>(delta.x / stroke.sx);
+    delta.y = static_cast<int32_t>(delta.y / stroke.sy);
     auto lineLength = mathLength(delta);
     auto angle = mathAtan(delta);
 
-    delta = {static_cast<SwCoord>(stroke.width), 0};
+    delta = {static_cast<int32_t>(stroke.width), 0};
     mathRotate(delta, angle + SW_ANGLE_PI2);
     SCALE(stroke, delta);
 
@@ -402,20 +384,11 @@ static void _lineTo(SwStroke& stroke, const SwPoint& to)
     }
 
     //now add a line segment to both the inside and outside paths
-    auto border = stroke.borders;
-    auto side = 1;
-
-    while (side >= 0) {
-        auto pt = to + delta;
-
+    for (int side = 0; side < 2; ++side) {
         //the ends of lineto borders are movable
-        _borderLineTo(border, pt, true);
-
+        _borderLineTo(stroke.borders[side], to + delta, true);
         delta.x = -delta.x;
         delta.y = -delta.y;
-
-        --side;
-        ++border;
     }
 
     stroke.angleIn = angle;
@@ -436,18 +409,31 @@ static void _cubicTo(SwStroke& stroke, const SwPoint& ctrl1, const SwPoint& ctrl
     arc[3] = stroke.center;
 
     while (arc >= bezStack) {
-        SwFixed angleIn, angleOut, angleMid;
+        int64_t angleIn, angleOut, angleMid;
 
         //initialize with current direction
         angleIn = angleOut = angleMid = stroke.angleIn;
 
-        if (arc < limit && !mathSmallCubic(arc, angleIn, angleMid, angleOut)) {
+        auto valid = mathCubicAngle(arc, angleIn, angleMid, angleOut);
+
+        //valid size
+        if (valid > 0 && arc < limit) {
             if (stroke.firstPt) stroke.angleIn = angleIn;
             mathSplitCubic(arc);
             arc += 3;
             continue;
         }
 
+        //ignoreable size
+        if (valid < 0 && arc == bezStack) {
+            stroke.center = to;
+
+            //round and square caps are expected to be drawn as a dot even for zero-length lines
+            if (stroke.firstPt && stroke.cap != StrokeCap::Butt) _firstSubPath(stroke, 0, 0);
+            return;
+        }
+
+        //small size
         if (firstArc) {
             firstArc = false;
             //process corner if necessary
@@ -476,75 +462,68 @@ static void _cubicTo(SwStroke& stroke, const SwPoint& ctrl1, const SwPoint& ctrl
         auto phi2 = mathMean(angleMid, angleOut);
         auto length1 = mathDivide(stroke.width, mathCos(theta1));
         auto length2 = mathDivide(stroke.width, mathCos(theta2));
-        SwFixed alpha0 = 0;
+        int64_t alpha0 = 0;
 
         //compute direction of original arc
         if (stroke.handleWideStrokes) {
             alpha0 = mathAtan(arc[0] - arc[3]);
         }
 
-        auto border = stroke.borders;
-        int32_t side = 0;
-
-        while (side < 2) {
+        for (int side = 0; side < 2; ++side) {
+            auto border = stroke.borders[side];
             auto rotate = SIDE_TO_ROTATE(side);
 
             //compute control points
-            SwPoint _ctrl1 = {static_cast<SwCoord>(length1), 0};
+            SwPoint _ctrl1 = {static_cast<int32_t>(length1), 0};
             mathRotate(_ctrl1, phi1 + rotate);
             SCALE(stroke, _ctrl1);
             _ctrl1 += arc[2];
 
-            SwPoint _ctrl2 = {static_cast<SwCoord>(length2), 0};
+            SwPoint _ctrl2 = {static_cast<int32_t>(length2), 0};
             mathRotate(_ctrl2, phi2 + rotate);
             SCALE(stroke, _ctrl2);
             _ctrl2 += arc[1];
 
             //compute end point
-            SwPoint _end = {static_cast<SwCoord>(stroke.width), 0};
-            mathRotate(_end, angleOut + rotate);
-            SCALE(stroke, _end);
-            _end += arc[0];
+            SwPoint end = {static_cast<int32_t>(stroke.width), 0};
+            mathRotate(end, angleOut + rotate);
+            SCALE(stroke, end);
+            end += arc[0];
 
             if (stroke.handleWideStrokes) {
                 /* determine whether the border radius is greater than the radius of
                    curvature of the original arc */
-                auto _start = border->pts[border->ptsCnt - 1];
-                auto alpha1 = mathAtan(_end - _start);
+                auto start = border->pts.last();
+                auto alpha1 = mathAtan(end - start);
 
                 //is the direction of the border arc opposite to that of the original arc?
                 if (abs(mathDiff(alpha0, alpha1)) > SW_ANGLE_PI / 2) {
 
                     //use the sine rule to find the intersection point
-                    auto beta = mathAtan(arc[3] - _start);
-                    auto gamma = mathAtan(arc[0] - _end);
-                    auto bvec = _end - _start;
+                    auto beta = mathAtan(arc[3] - start);
+                    auto gamma = mathAtan(arc[0] - end);
+                    auto bvec = end - start;
                     auto blen = mathLength(bvec);
                     auto sinA = abs(mathSin(alpha1 - gamma));
                     auto sinB = abs(mathSin(beta - gamma));
                     auto alen = mathMulDiv(blen, sinA, sinB);
 
-                    SwPoint delta = {static_cast<SwCoord>(alen), 0};
+                    SwPoint delta = {static_cast<int32_t>(alen), 0};
                     mathRotate(delta, beta);
-                    delta += _start;
+                    delta += start;
 
                     //circumnavigate the negative sector backwards
                     border->movable = false;
                     _borderLineTo(border, delta, false);
-                    _borderLineTo(border, _end, false);
-                    _borderCubicTo(border, _ctrl2, _ctrl1, _start);
+                    _borderLineTo(border, end, false);
+                    _borderCubicTo(border, _ctrl2, _ctrl1, start);
 
                     //and then move to the endpoint
-                    _borderLineTo(border, _end, false);
-
-                    ++side;
-                    ++border;
+                    _borderLineTo(border, end, false);
                     continue;
                 }
             }
-            _borderCubicTo(border, _ctrl1, _ctrl2, _end);
-            ++side;
-            ++border;
+            _borderCubicTo(border, _ctrl1, _ctrl2, end);
         }
         arc -= 3;
         stroke.angleIn = angleOut;
@@ -553,53 +532,49 @@ static void _cubicTo(SwStroke& stroke, const SwPoint& ctrl1, const SwPoint& ctrl
 }
 
 
-static void _addCap(SwStroke& stroke, SwFixed angle, int32_t side)
+static void _addCap(SwStroke& stroke, int64_t angle, int32_t side)
 {
     if (stroke.cap == StrokeCap::Square) {
         auto rotate = SIDE_TO_ROTATE(side);
-        auto border = stroke.borders + side;
+        auto border = stroke.borders[side];
 
-        SwPoint delta = {static_cast<SwCoord>(stroke.width), 0};
+        SwPoint delta = {static_cast<int32_t>(stroke.width), 0};
         mathRotate(delta, angle);
         SCALE(stroke, delta);
 
-        SwPoint delta2 = {static_cast<SwCoord>(stroke.width), 0};
+        SwPoint delta2 = {static_cast<int32_t>(stroke.width), 0};
         mathRotate(delta2, angle + rotate);
         SCALE(stroke, delta2);
         delta += stroke.center + delta2;
 
         _borderLineTo(border, delta, false);
 
-        delta = {static_cast<SwCoord>(stroke.width), 0};
+        delta = {static_cast<int32_t>(stroke.width), 0};
         mathRotate(delta, angle);
         SCALE(stroke, delta);
 
-        delta2 = {static_cast<SwCoord>(stroke.width), 0};
+        delta2 = {static_cast<int32_t>(stroke.width), 0};
         mathRotate(delta2, angle - rotate);
         SCALE(stroke, delta2);
         delta += delta2 + stroke.center;
 
         _borderLineTo(border, delta, false);
-
     } else if (stroke.cap == StrokeCap::Round) {
-
         stroke.angleIn = angle;
         stroke.angleOut = angle + SW_ANGLE_PI;
         _arcTo(stroke, side);
-        return;
-
     } else {  //Butt
         auto rotate = SIDE_TO_ROTATE(side);
-        auto border = stroke.borders + side;
+        auto border = stroke.borders[side];
 
-        SwPoint delta = {static_cast<SwCoord>(stroke.width), 0};
+        SwPoint delta = {static_cast<int32_t>(stroke.width), 0};
         mathRotate(delta, angle + rotate);
         SCALE(stroke, delta);
         delta += stroke.center;
 
         _borderLineTo(border, delta, false);
 
-        delta = {static_cast<SwCoord>(stroke.width), 0};
+        delta = {static_cast<int32_t>(stroke.width), 0};
         mathRotate(delta, angle - rotate);
         SCALE(stroke, delta);
         delta += stroke.center;
@@ -611,21 +586,20 @@ static void _addCap(SwStroke& stroke, SwFixed angle, int32_t side)
 
 static void _addReverseLeft(SwStroke& stroke, bool opened)
 {
-    auto right = stroke.borders + 0;
-    auto left = stroke.borders + 1;
-    auto newPts = left->ptsCnt - left->start;
+    auto right = stroke.borders[0];
+    auto left = stroke.borders[1];
+    auto newPts = left->pts.count - left->start;
 
     if (newPts <= 0) return;
 
     _growBorder(right, newPts);
 
-    auto dstPt = right->pts + right->ptsCnt;
-    auto dstTag = right->tags + right->ptsCnt;
-    auto srcPt = left->pts + left->ptsCnt - 1;
-    auto srcTag = left->tags + left->ptsCnt - 1;
+    auto dstTag = right->tags + right->pts.count;
+    auto srcPt = left->pts.end() - 1;
+    auto srcTag = left->tags + left->pts.count - 1;
 
-    while (srcPt >= left->pts + left->start) {
-        *dstPt = *srcPt;
+    while (srcPt >= left->pts.data + left->start) {
+        right->pts.push(*srcPt);
         *dstTag = *srcTag;
 
         if (opened) {
@@ -638,12 +612,10 @@ static void _addReverseLeft(SwStroke& stroke, bool opened)
         }
         --srcPt;
         --srcTag;
-        ++dstPt;
         ++dstTag;
     }
 
-    left->ptsCnt = left->start;
-    right->ptsCnt += newPts;
+    left->pts.count = left->start;
     right->movable = false;
     left->movable = false;
 }
@@ -662,7 +634,7 @@ static void _beginSubPath(SwStroke& stroke, const SwPoint& to, bool closed)
     /* Determine if we need to check whether the border radius is greater
        than the radius of curvature of a curve, to handle this case specially.
        This is only required if bevel joins or butt caps may be created because
-       round & miter joins and round & square caps cover the nagative sector
+       round & miter joins and round & square caps cover the negative sector
        created with wide strokes. */
     if ((stroke.join != StrokeJoin::Round) || (!stroke.closedSubPath && stroke.cap == StrokeCap::Butt))
         stroke.handleWideStrokes = true;
@@ -697,10 +669,10 @@ static void _endSubPath(SwStroke& stroke)
             _outside(stroke, 1 - inside, stroke.subPathLineLength);   //outside
         }
 
-        _borderClose(stroke.borders + 0, false);
-        _borderClose(stroke.borders + 1, true);
+        _borderClose(stroke.borders[0], false);
+        _borderClose(stroke.borders[1], true);
     } else {
-        auto right = stroke.borders;
+        auto right = stroke.borders[0];
 
         /* all right, this is an opened path, we need to add a cap between
            right & left, add the reverse of left, then add a final cap
@@ -715,72 +687,28 @@ static void _endSubPath(SwStroke& stroke)
         _addCap(stroke, stroke.subPathAngle + SW_ANGLE_PI, 0);
 
         /* now end the right subpath accordingly. The left one is rewind
-           and deosn't need further processing */
+           and doesn't need further processing */
         _borderClose(right, false);
     }
 }
 
 
-static void _getCounts(SwStrokeBorder* border, uint32_t& ptsCnt, uint32_t& cntrsCnt)
-{
-    auto count = border->ptsCnt;
-    auto tags = border->tags;
-    uint32_t _ptsCnt = 0;
-    uint32_t _cntrsCnt = 0;
-    bool inCntr = false;
-
-    while (count > 0) {
-        if (tags[0] & SW_STROKE_TAG_BEGIN) {
-            if (inCntr) goto fail;
-            inCntr = true;
-        } else if (!inCntr) goto fail;
-
-        if (tags[0] & SW_STROKE_TAG_END) {
-            inCntr = false;
-            ++_cntrsCnt;
-        }
-        --count;
-        ++_ptsCnt;
-        ++tags;
-    }
-
-    if (inCntr) goto fail;
-
-    ptsCnt = _ptsCnt;
-    cntrsCnt = _cntrsCnt;
-
-    return;
-
-fail:
-    ptsCnt = 0;
-    cntrsCnt = 0;
-}
-
-
 static void _exportBorderOutline(const SwStroke& stroke, SwOutline* outline, uint32_t side)
 {
-    auto border = stroke.borders + side;
-    if (border->ptsCnt == 0) return;
+    auto border = stroke.borders[side];
+    if (border->pts.empty()) return;
 
-    memcpy(outline->pts.data + outline->pts.count, border->pts, border->ptsCnt * sizeof(SwPoint));
-
-    auto cnt = border->ptsCnt;
     auto src = border->tags;
-    auto tags = outline->types.data + outline->types.count;
     auto idx = outline->pts.count;
 
-    while (cnt > 0) {
-        if (*src & SW_STROKE_TAG_POINT) *tags = SW_CURVE_TYPE_POINT;
-        else if (*src & SW_STROKE_TAG_CUBIC) *tags = SW_CURVE_TYPE_CUBIC;
-        else TVGERR("SW_ENGINE", "Invalid stroke tag was given! = %d", *src);
+    ARRAY_FOREACH(pts, border->pts) {
+        if (*src & SW_STROKE_TAG_POINT) outline->types.push(SW_CURVE_TYPE_POINT);
+        else if (*src & SW_STROKE_TAG_CUBIC) outline->types.push(SW_CURVE_TYPE_CUBIC);
         if (*src & SW_STROKE_TAG_END) outline->cntrs.push(idx);
         ++src;
-        ++tags;
         ++idx;
-        --cnt;
     }
-    outline->pts.count += border->ptsCnt;
-    outline->types.count += border->ptsCnt;
+    outline->pts.push(border->pts);
 }
 
 
@@ -792,50 +720,38 @@ void strokeFree(SwStroke* stroke)
 {
     if (!stroke) return;
 
-    //free borders
-    if (stroke->borders[0].pts) free(stroke->borders[0].pts);
-    if (stroke->borders[0].tags) free(stroke->borders[0].tags);
-    if (stroke->borders[1].pts) free(stroke->borders[1].pts);
-    if (stroke->borders[1].tags) free(stroke->borders[1].tags);
-
     fillFree(stroke->fill);
     stroke->fill = nullptr;
 
-    free(stroke);
+    tvg::free(stroke);
 }
 
 
-void strokeReset(SwStroke* stroke, const RenderShape* rshape, const Matrix* transform)
+void strokeReset(SwStroke* stroke, const RenderShape* rshape, const Matrix& transform, SwMpool* mpool, unsigned tid)
 {
-    if (transform) {
-        stroke->sx = sqrtf(powf(transform->e11, 2.0f) + powf(transform->e21, 2.0f));
-        stroke->sy = sqrtf(powf(transform->e12, 2.0f) + powf(transform->e22, 2.0f));
-    } else {
-        stroke->sx = stroke->sy = 1.0f;
-    }
-
+    stroke->sx = sqrtf(powf(transform.e11, 2.0f) + powf(transform.e21, 2.0f));
+    stroke->sy = sqrtf(powf(transform.e12, 2.0f) + powf(transform.e22, 2.0f));
     stroke->width = HALF_STROKE(rshape->strokeWidth());
     stroke->cap = rshape->strokeCap();
-    stroke->miterlimit = static_cast<SwFixed>(rshape->strokeMiterlimit()) << 16;
+    stroke->miterlimit = static_cast<int64_t>(rshape->strokeMiterlimit() * 65536.0f);
 
     //Save line join: it can be temporarily changed when stroking curves...
     stroke->joinSaved = stroke->join = rshape->strokeJoin();
 
-    stroke->borders[0].ptsCnt = 0;
-    stroke->borders[0].start = -1;
-    stroke->borders[1].ptsCnt = 0;
-    stroke->borders[1].start = -1;
+    stroke->borders[0] = mpoolReqStrokeLBorder(mpool, tid);
+    stroke->borders[1] = mpoolReqStrokeRBorder(mpool, tid);
 }
 
 
-bool strokeParseOutline(SwStroke* stroke, const SwOutline& outline)
+bool strokeParseOutline(SwStroke* stroke, const SwOutline& outline, SwMpool* mpool, unsigned tid)
 {
     uint32_t first = 0;
     uint32_t i = 0;
 
-    for (auto cntr = outline.cntrs.begin(); cntr < outline.cntrs.end(); ++cntr, ++i) {
-        auto last = *cntr;           //index of last point in contour
+    ARRAY_FOREACH(p, outline.cntrs) {
+        auto last = *p;           //index of last point in contour
         auto limit = outline.pts.data + last;
+        ++i;
 
         //Skip empty points
         if (last <= first) {
@@ -850,31 +766,25 @@ bool strokeParseOutline(SwStroke* stroke, const SwOutline& outline)
 
         //A contour cannot start with a cubic control point
         if (type == SW_CURVE_TYPE_CUBIC) return false;
+        ++types;
 
-        auto closed =  outline.closed.data ? outline.closed.data[i]: false;
+        auto closed =  outline.closed.data ? outline.closed.data[i - 1]: false;
 
         _beginSubPath(*stroke, start, closed);
 
         while (pt < limit) {
-            ++pt;
-            ++types;
-
-            //emit a signel line_to
+            //emit a single line_to
             if (types[0] == SW_CURVE_TYPE_POINT) {
+                ++pt;
+                ++types;
                 _lineTo(*stroke, *pt);
             //types cubic
             } else {
-                if (pt + 1 > limit || types[1] != SW_CURVE_TYPE_CUBIC) return false;
-
-                pt += 2;
-                types += 2;
-
-                if (pt <= limit) {
-                    _cubicTo(*stroke, pt[-2], pt[-1], pt[0]);
-                    continue;
-                }
-                _cubicTo(*stroke, pt[-2], pt[-1], start);
-                goto close;
+                pt += 3;
+                types += 3;
+                if (pt <= limit) _cubicTo(*stroke, pt[-2], pt[-1], pt[0]);
+                else if (pt - 1 == limit) _cubicTo(*stroke, pt[-2], pt[-1], start);
+                else goto close;
             }
         }
     close:
@@ -887,18 +797,11 @@ bool strokeParseOutline(SwStroke* stroke, const SwOutline& outline)
 
 SwOutline* strokeExportOutline(SwStroke* stroke, SwMpool* mpool, unsigned tid)
 {
-    uint32_t count1, count2, count3, count4;
-
-    _getCounts(stroke->borders + 0, count1, count2);
-    _getCounts(stroke->borders + 1, count3, count4);
-
-    auto ptsCnt = count1 + count3;
-    auto cntrsCnt = count2 + count4;
-
-    auto outline = mpoolReqStrokeOutline(mpool, tid);
-    outline->pts.reserve(ptsCnt);
-    outline->types.reserve(ptsCnt);
-    outline->cntrs.reserve(cntrsCnt);
+    auto reserve = stroke->borders[0]->pts.count + stroke->borders[1]->pts.count;
+    auto outline = mpoolReqOutline(mpool, tid);
+    outline->pts.reserve(reserve);
+    outline->types.reserve(reserve);
+    outline->fillRule = FillRule::NonZero;
 
     _exportBorderOutline(*stroke, outline, 0);  //left
     _exportBorderOutline(*stroke, outline, 1);  //right
