@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2020 - 2024 the ThorVG project. All rights reserved.
+ * Copyright (c) 2020 - 2026 ThorVG project. All rights reserved.
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -20,10 +20,9 @@
  * SOFTWARE.
  */
 
-#include <string.h>
-
 #include <atomic>
 #include "tvgInlist.h"
+#include "tvgStr.h"
 #include "tvgLoader.h"
 #include "tvgLock.h"
 
@@ -35,10 +34,6 @@
     #include "tvgPngLoader.h"
 #endif
 
-#ifdef THORVG_TVG_LOADER_SUPPORT
-    #include "tvgTvgLoader.h"
-#endif
-
 #ifdef THORVG_JPG_LOADER_SUPPORT
     #include "tvgJpgLoader.h"
 #endif
@@ -71,10 +66,10 @@ uintptr_t HASH_KEY(const char* data)
 atomic<ColorSpace> ImageLoader::cs{ColorSpace::ARGB8888};
 
 static Key _key;
-static Inlist<LoadModule> _activeLoaders;
+static Inlist<tvg::LoadModule> _activeLoaders;
 
 
-static LoadModule* _find(FileType type)
+static tvg::LoadModule* _find(FileType type)
 {
     switch(type) {
         case FileType::Png: {
@@ -92,12 +87,6 @@ static LoadModule* _find(FileType type)
         case FileType::Webp: {
 #ifdef THORVG_WEBP_LOADER_SUPPORT
             return new WebpLoader;
-#endif
-            break;
-        }
-        case FileType::Tvg: {
-#ifdef THORVG_TVG_LOADER_SUPPORT
-            return new TvgLoader;
 #endif
             break;
         }
@@ -113,7 +102,7 @@ static LoadModule* _find(FileType type)
 #endif
             break;
         }
-        case FileType::Lottie: {
+        case FileType::Lot: {
 #ifdef THORVG_LOTTIE_LOADER_SUPPORT
             return new LottieLoader;
 #endif
@@ -131,10 +120,6 @@ static LoadModule* _find(FileType type)
 #ifdef THORVG_LOG_ENABLED
     const char *format;
     switch(type) {
-        case FileType::Tvg: {
-            format = "TVG";
-            break;
-        }
         case FileType::Svg: {
             format = "SVG";
             break;
@@ -143,8 +128,8 @@ static LoadModule* _find(FileType type)
             format = "TTF";
             break;
         }
-        case FileType::Lottie: {
-            format = "lottie(json)";
+        case FileType::Lot: {
+            format = "LOT";
             break;
         }
         case FileType::Raw: {
@@ -175,78 +160,75 @@ static LoadModule* _find(FileType type)
 
 
 #ifdef THORVG_FILE_IO_SUPPORT
-static LoadModule* _findByPath(const string& path)
+static tvg::LoadModule* _findByPath(const char* filename)
 {
-    auto ext = path.substr(path.find_last_of(".") + 1);
-    if (!ext.compare("tvg")) return _find(FileType::Tvg);
-    if (!ext.compare("svg")) return _find(FileType::Svg);
-    if (!ext.compare("json")) return _find(FileType::Lottie);
-    if (!ext.compare("png")) return _find(FileType::Png);
-    if (!ext.compare("jpg")) return _find(FileType::Jpg);
-    if (!ext.compare("webp")) return _find(FileType::Webp);
-    if (!ext.compare("ttf") || !ext.compare("ttc")) return _find(FileType::Ttf);
-    if (!ext.compare("otf") || !ext.compare("otc")) return _find(FileType::Ttf);
+    auto ext = fileext(filename);
+    if (!ext) return nullptr;
+
+    if (!strcmp(ext, "svg")) return _find(FileType::Svg);
+    if (!strcmp(ext, "lot") || !strcmp(ext, "json")) return _find(FileType::Lot);
+    if (!strcmp(ext, "png")) return _find(FileType::Png);
+    if (!strcmp(ext, "jpg")) return _find(FileType::Jpg);
+    if (!strcmp(ext, "webp")) return _find(FileType::Webp);
+    if (!strcmp(ext, "ttf") || !strcmp(ext, "ttc")) return _find(FileType::Ttf);
+    if (!strcmp(ext, "otf") || !strcmp(ext, "otc")) return _find(FileType::Ttf);
     return nullptr;
 }
 #endif
 
 
-static FileType _convert(const string& mimeType)
+static FileType _convert(const char* mimeType)
 {
+    if (!mimeType) return FileType::Unknown;
+
     auto type = FileType::Unknown;
 
-    if (mimeType == "tvg") type = FileType::Tvg;
-    else if (mimeType == "svg" || mimeType == "svg+xml") type = FileType::Svg;
-    else if (mimeType == "ttf" || mimeType == "otf") type = FileType::Ttf;
-    else if (mimeType == "lottie") type = FileType::Lottie;
-    else if (mimeType == "raw") type = FileType::Raw;
-    else if (mimeType == "png") type = FileType::Png;
-    else if (mimeType == "jpg" || mimeType == "jpeg") type = FileType::Jpg;
-    else if (mimeType == "webp") type = FileType::Webp;
-    else TVGLOG("RENDERER", "Given mimetype is unknown = \"%s\".", mimeType.c_str());
+    if (!strcmp(mimeType, "svg") || !strcmp(mimeType, "svg+xml")) type = FileType::Svg;
+    else if (!strcmp(mimeType, "ttf") || !strcmp(mimeType, "otf")) type = FileType::Ttf;
+    else if (!strcmp(mimeType, "lot") || !strcmp(mimeType, "lottie+json")) type = FileType::Lot;
+    else if (!strcmp(mimeType, "raw")) type = FileType::Raw;
+    else if (!strcmp(mimeType, "png")) type = FileType::Png;
+    else if (!strcmp(mimeType, "jpg") || !strcmp(mimeType, "jpeg")) type = FileType::Jpg;
+    else if (!strcmp(mimeType, "webp")) type = FileType::Webp;
+    else TVGLOG("RENDERER", "Given mimetype is unknown = \"%s\".", mimeType);
 
     return type;
 }
 
 
-static LoadModule* _findByType(const string& mimeType)
+static tvg::LoadModule* _findByType(const char* mimeType)
 {
     return _find(_convert(mimeType));
 }
 
 
-static LoadModule* _findFromCache(const string& path)
+static tvg::LoadModule* _findFromCache(const char* filename)
 {
     ScopedLock lock(_key);
-
-    auto loader = _activeLoaders.head;
-
-    while (loader) {
-        if (loader->pathcache && !strcmp(loader->hashpath, path.c_str())) {
+    INLIST_FOREACH(_activeLoaders, loader) {
+        if (loader->cached && loader->hashpath && !strcmp(loader->hashpath, filename)) {
             ++loader->sharing;
             return loader;
         }
-        loader = loader->next;
     }
     return nullptr;
 }
 
 
-static LoadModule* _findFromCache(const char* data, uint32_t size, const string& mimeType)
+static tvg::LoadModule* _findFromCache(const char* data, uint32_t size, const char* mimeType)
 {
     auto type = _convert(mimeType);
     if (type == FileType::Unknown) return nullptr;
 
     auto key = HASH_KEY(data);
+
     ScopedLock lock(_key);
 
-    auto loader = _activeLoaders.head;
-    while (loader) {
+    INLIST_FOREACH(_activeLoaders, loader) {
         if (loader->type == type && loader->hashkey == key) {
             ++loader->sharing;
             return loader;
         }
-        loader = loader->next;
     }
     return nullptr;
 }
@@ -265,19 +247,12 @@ bool LoaderMgr::init()
 
 bool LoaderMgr::term()
 {
-    auto loader = _activeLoaders.head;
-
     //clean up the remained font loaders which is globally used.
-    while (loader) {
-        if (loader->type != FileType::Ttf) {
-            loader = loader->next;
-            continue;
-        }
+    INLIST_SAFE_FOREACH(_activeLoaders, loader) {
+        if (loader->type != FileType::Ttf) continue;
         auto ret = loader->close();
-        auto tmp = loader;
-        loader = loader->next;
-        _activeLoaders.remove(tmp);
-        if (ret) delete(tmp);
+        _activeLoaders.remove(loader);
+        if (ret) delete(loader);
     }
     return true;
 }
@@ -288,7 +263,7 @@ bool LoaderMgr::retrieve(LoadModule* loader)
     if (!loader) return false;
 
     if (loader->close()) {
-        if (loader->cached()) {
+        if (loader->cached) {
             _activeLoaders.remove(loader);
         }
         delete(loader);
@@ -297,25 +272,24 @@ bool LoaderMgr::retrieve(LoadModule* loader)
 }
 
 
-LoadModule* LoaderMgr::loader(const string& path, bool* invalid)
+tvg::LoadModule* LoaderMgr::loader(const char* filename, bool* invalid)
 {
 #ifdef THORVG_FILE_IO_SUPPORT
     *invalid = false;
 
-    //TODO: svg & lottie is not sharable.
+    //TODO: make lottie sharable.
     auto allowCache = true;
-    auto ext = path.substr(path.find_last_of(".") + 1);
-    if (!ext.compare("svg") || !ext.compare("json")) allowCache = false;
+    auto ext = fileext(filename);
+    if (ext && (!strcmp(ext, "json") || !strcmp(ext, "lot"))) allowCache = false;
 
     if (allowCache) {
-        if (auto loader = _findFromCache(path)) return loader;
+        if (auto loader = _findFromCache(filename)) return loader;
     }
 
-    if (auto loader = _findByPath(path)) {
-        if (loader->open(path)) {
+    if (auto loader = _findByPath(filename)) {
+        if (loader->open(filename)) {
             if (allowCache) {
-                loader->hashpath = strdup(path.c_str());
-                loader->pathcache = true;
+                loader->cache(duplicate(filename));
                 {
                     ScopedLock lock(_key);
                     _activeLoaders.back(loader);
@@ -328,10 +302,9 @@ LoadModule* LoaderMgr::loader(const string& path, bool* invalid)
     //Unknown MimeType. Try with the candidates in the order
     for (int i = 0; i < static_cast<int>(FileType::Raw); i++) {
         if (auto loader = _find(static_cast<FileType>(i))) {
-            if (loader->open(path)) {
+            if (loader->open(filename)) {
                 if (allowCache) {
-                    loader->hashpath = strdup(path.c_str());
-                    loader->pathcache = true;
+                    loader->cache(duplicate(filename));
                     {
                         ScopedLock lock(_key);
                         _activeLoaders.back(loader);
@@ -348,37 +321,22 @@ LoadModule* LoaderMgr::loader(const string& path, bool* invalid)
 }
 
 
-bool LoaderMgr::retrieve(const string& path)
+bool LoaderMgr::retrieve(const char* filename)
 {
-    return retrieve(_findFromCache(path));
+    return retrieve(_findFromCache(filename));
 }
 
 
-LoadModule* LoaderMgr::loader(const char* key)
-{
-    auto loader = _activeLoaders.head;
-
-    while (loader) {
-        if (loader->pathcache && strstr(loader->hashpath, key)) {
-            ++loader->sharing;
-            return loader;
-        }
-        loader = loader->next;
-    }
-    return nullptr;
-}
-
-
-LoadModule* LoaderMgr::loader(const char* data, uint32_t size, const string& mimeType, bool copy)
+tvg::LoadModule* LoaderMgr::loader(const char* data, uint32_t size, const char* mimeType, const char* rpath, bool copy)
 {
     //Note that users could use the same data pointer with the different content.
     //Thus caching is only valid for shareable.
     auto allowCache = !copy;
 
-    //TODO: lottie is not sharable.
+    //TODO: make lottie shareable.
     if (allowCache) {
         auto type = _convert(mimeType);
-        if (type == FileType::Lottie) allowCache = false;
+        if (type == FileType::Lot) allowCache = false;
     }
 
     if (allowCache) {
@@ -386,17 +344,17 @@ LoadModule* LoaderMgr::loader(const char* data, uint32_t size, const string& mim
     }
 
     //Try with the given MimeType
-    if (!mimeType.empty()) {
+    if (mimeType) {
         if (auto loader = _findByType(mimeType)) {
-            if (loader->open(data, size, copy)) {
+            if (loader->open(data, size, rpath, copy)) {
                 if (allowCache) {
-                    loader->hashkey = HASH_KEY(data);
+                    loader->cache(HASH_KEY(data));
                     ScopedLock lock(_key);
                     _activeLoaders.back(loader);
                 }
                 return loader;
             } else {
-                TVGLOG("LOADER", "Given mimetype \"%s\" seems incorrect or not supported.", mimeType.c_str());
+                TVGLOG("LOADER", "Given mimetype \"%s\" seems incorrect or not supported.", mimeType);
                 delete(loader);
             }
         }
@@ -405,9 +363,9 @@ LoadModule* LoaderMgr::loader(const char* data, uint32_t size, const string& mim
     for (int i = 0; i < static_cast<int>(FileType::Raw); i++) {
         auto loader = _find(static_cast<FileType>(i));
         if (loader) {
-            if (loader->open(data, size, copy)) {
+            if (loader->open(data, size, rpath, copy)) {
                 if (allowCache) {
-                    loader->hashkey = HASH_KEY(data);
+                    loader->cache(HASH_KEY(data));
                     ScopedLock lock(_key);
                     _activeLoaders.back(loader);
                 }
@@ -420,7 +378,7 @@ LoadModule* LoaderMgr::loader(const char* data, uint32_t size, const string& mim
 }
 
 
-LoadModule* LoaderMgr::loader(const uint32_t *data, uint32_t w, uint32_t h, bool copy)
+tvg::LoadModule* LoaderMgr::loader(const uint32_t *data, uint32_t w, uint32_t h, ColorSpace cs, bool copy)
 {
     //Note that users could use the same data pointer with the different content.
     //Thus caching is only valid for shareable.
@@ -431,9 +389,9 @@ LoadModule* LoaderMgr::loader(const uint32_t *data, uint32_t w, uint32_t h, bool
 
     //function is dedicated for raw images only
     auto loader = new RawLoader;
-    if (loader->open(data, w, h, copy)) {
+    if (loader->open(data, w, h, cs, copy)) {
         if (!copy) {
-            loader->hashkey = HASH_KEY((const char*)data);
+            loader->cache(HASH_KEY((const char*)data));
             ScopedLock lock(_key);
             _activeLoaders.back(loader);
         }
@@ -445,17 +403,17 @@ LoadModule* LoaderMgr::loader(const uint32_t *data, uint32_t w, uint32_t h, bool
 
 
 //loads fonts from memory - loader is cached (regardless of copy value) in order to access it while setting font
-LoadModule* LoaderMgr::loader(const char* name, const char* data, uint32_t size, TVG_UNUSED const string& mimeType, bool copy)
+tvg::LoadModule* LoaderMgr::loader(const char* name, const char* data, uint32_t size, TVG_UNUSED const char* mimeType, bool copy)
 {
 #ifdef THORVG_TTF_LOADER_SUPPORT
     //TODO: add check for mimetype ?
-    if (auto loader = _findFromCache(name)) return loader;
+    if (auto loader = font(name)) return loader;
 
     //function is dedicated for ttf loader (the only supported font loader)
     auto loader = new TtfLoader;
-    if (loader->open(data, size, copy)) {
-        loader->hashpath = strdup(name);
-        loader->pathcache = true;
+    if (loader->open(data, size, "", copy)) {
+        loader->name = duplicate(name);
+        loader->cached = true;  //force it.
         ScopedLock lock(_key);
         _activeLoaders.back(loader);
         return loader;
@@ -465,4 +423,31 @@ LoadModule* LoaderMgr::loader(const char* name, const char* data, uint32_t size,
     delete(loader);
 #endif
     return nullptr;
-}
\ No newline at end of file
+}
+
+
+tvg::LoadModule* LoaderMgr::font(const char* name)
+{
+    ScopedLock lock(_key);
+    INLIST_FOREACH(_activeLoaders, loader) {
+        if (loader->type != FileType::Ttf) continue;
+        if (loader->cached && tvg::equal(name, static_cast<FontLoader*>(loader)->name)) {
+            ++loader->sharing;
+            return loader;
+        }
+    }
+    return nullptr;
+}
+
+
+tvg::LoadModule* LoaderMgr::anyfont()
+{
+    ScopedLock lock(_key);
+    INLIST_FOREACH(_activeLoaders, loader) {
+        if (loader->cached && loader->type == FileType::Ttf) {
+            ++loader->sharing;
+            return loader;
+        }
+    }
+    return nullptr;
+}
