@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021 - 2024 the ThorVG project. All rights reserved.
+ * Copyright (c) 2021 - 2026 ThorVG project. All rights reserved.
 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -30,19 +30,19 @@ using namespace tvg;
 
 TEST_CASE("Basic initialization", "[tvgInitializer]")
 {
-    REQUIRE(Initializer::init(CanvasEngine::Sw, 0) == Result::Success);
-    REQUIRE(Initializer::term(CanvasEngine::Sw) == Result::Success);
+    REQUIRE(Initializer::init() == Result::Success);
+    REQUIRE(Initializer::term() == Result::Success);
 }
 
 TEST_CASE("Multiple initialization", "[tvgInitializer]")
 {
-    REQUIRE(Initializer::init(CanvasEngine::Sw, 0) == Result::Success);
-    REQUIRE(Initializer::init(CanvasEngine::Sw, 0) == Result::Success);
-    REQUIRE(Initializer::term(CanvasEngine::Sw) == Result::Success);
+    REQUIRE(Initializer::init() == Result::Success);
+    REQUIRE(Initializer::init() == Result::Success);
+    REQUIRE(Initializer::term() == Result::Success);
 
-    REQUIRE(Initializer::init(CanvasEngine::Sw, 0) == Result::Success);
-    REQUIRE(Initializer::term(CanvasEngine::Sw) == Result::Success);
-    REQUIRE(Initializer::term(CanvasEngine::Sw) == Result::Success);
+    REQUIRE(Initializer::init() == Result::Success);
+    REQUIRE(Initializer::term() == Result::Success);
+    REQUIRE(Initializer::term() == Result::Success);
 }
 
 TEST_CASE("Version", "[tvgInitializer]")
@@ -57,10 +57,5 @@ TEST_CASE("Version", "[tvgInitializer]")
 
 TEST_CASE("Negative termination", "[tvgInitializer]")
 {
-    REQUIRE(Initializer::term(CanvasEngine::Sw) == Result::InsufficientCondition);
-}
-
-TEST_CASE("Invalid engine", "[tvgInitializer]")
-{
-    REQUIRE(Initializer::init(CanvasEngine(64), 0) == Result::NonSupport);
-}
+    REQUIRE(Initializer::term() == Result::InsufficientCondition);
+}
\ No newline at end of file
